/**********************************************************************
 *<
	FILE: PW_PART.CPP

	DESCRIPTION: All purpose particle system

	CREATED BY: Peter Watje

	HISTORY: created 13 August 1996

 *>	Copyright (c) 1996, All Rights Reserved.
 **********************************************************************/



#include "prim.h"
#include "iparamm.h"
#include "Simpobj.h"
#include "texutil.h"
#include "istdplug.h"
#include "decomp.h"
#include "trig.h"
#include "bmmlib.h"
#include "splshape.h"
#include "simpspl.h"

#include "hair.h"


//extern ParamBlockDescID descVer4[];
//extern ParamVersionDesc versions[];
//extern ParamVersionDesc curVersionSnow;
//extern ParamUIDesc descParamSnow[];


static SnowClassDesc snowDesc;
ClassDesc* GetSnowDesc() {return &snowDesc;}

IParamMap *GenParticle::pmapParam;



class BirthRateDimension : public ParamDimension {
	public:
		DimType DimensionType() {return DIM_CUSTOM;}
		float Convert(float value) {return value*(float)GetTicksPerFrame();}
		float UnConvert(float value) {return value/(float)GetTicksPerFrame();}
	};
static BirthRateDimension theBirthRateDim;





//need to put in time parameter

float GenParticle::CompParticleSize(TimeValue t,
		TimeValue age, TimeValue life, float size)
	{

	float u;
	if (WidthGraphNode == NULL )
		{
		u = float(age+1.0f)/float(life);
		return (1.0f-u) * size;
		}
		else
		{
		float per;
 		u = float(age+1.0f)/float(life);
		per = GetGraph(WIDTHOBJECT_REF,t, u);
		return per  * size;
		}

	}






int	GenParticle::GetParticleCount()
	{
	int c;
	int d;
	pblock->GetValue(PB_SEGCOUNT,0,d,FOREVER);
	if (TestAFlag(A_RENDER)) 
		{
		pblock->GetValue(PB_RNDPARTICLES,0,c,FOREVER);
		d = d*2;
		}
	else
		{
		pblock->GetValue(PB_VPTPARTICLES,0,c,FOREVER);
		}
	

	return c*d;
	}

int	GenParticle::GetStrandCount()
	{
	int c;
//	int d;
	if (TestAFlag(A_RENDER)) pblock->GetValue(PB_RNDPARTICLES,0,c,FOREVER);
	else pblock->GetValue(PB_VPTPARTICLES,0,c,FOREVER);
	return c;
	}


int GenParticle::CountLive()
	{
	int c=0;
	for (int i=0; i<parts.Count(); i++) {
		if (parts.Alive(i)) c++;
		}
	return c;
	}

void GenParticle::ComputeParticleStart(TimeValue t0,INode *node)
	{
	int c = GetParticleCount();	
	parts.SetCount(c,PARTICLE_VELS|PARTICLE_AGES);
	for (int i=0; i<parts.Count(); i++) {
		parts.ages[i] = -1;
		}
	tvalid = t0;
	valid  = TRUE;
	}


#define VEL_SCALE	(-0.01f*1200.0f/float(TIME_TICKSPERSEC))
#define VAR_SCALE	(0.01f*1200.0f/float(TIME_TICKSPERSEC))

void GenParticle::BirthParticle(INode *node,TimeValue bt,int index, Matrix3 stm)
	{
	float initVel;
//	, var;
	float width, height;
//	,rad,angle;
	int TPerFrame;
	int emmit_shape;
	TimeValue Offset = 0;
//	float s,t;
//	int streamvel;
//	float per;
	char Prompt[80];
	int secondary;

	if ((Tempip != NULL) && ((index%1000)==0))
		{
		sprintf(Prompt,"Birthing Particle %d at  %d",index,bt);
		Tempip->PushPrompt(_T(Prompt));
		}

	Point3 pos, vel;

	TPerFrame = GetTicksPerFrame();

//	Offset = (TimeValue) ((float)TPerFrame*float(rand())/float(RAND_MAX));




	Matrix3 tm;// = node->GetObjTMBeforeWSM(bt);
//	Matrix3 stm;// = node->GetObjTMBeforeWSM(bt-TPerFrame);

	pblock->GetValue(PB_EMITTERWIDTH,bt,width,FOREVER);
	pblock->GetValue(PB_EMITTERHEIGHT,bt,height,FOREVER);
	pblock->GetValue(PB_SPEED,bt,initVel,FOREVER);

	emmit_shape = 6;





	parts.ages[index] = 0;




	tm = ObjectNode->GetObjTMAfterWSM(bt);


	pblock->GetValue(PB_SECONDARY,bt,secondary,FOREVER);

//put in list
	LastParticle = index;
		int i;
		i = index % NumberStrands;
		if (i >= Starting.Count())
			i = Starting.Count()-1;
	
	parts[index] = Starting[i].pos * tm;
	parts.vels[index] = VectorTransform(tm,Starting[i].vel*initVel);
//VectorTransform(tm,Starting[j%NumberStrands].vel)*initVel

//	if (secondary)
//		SecondaryMotion[index] = parts[index]-(Starting[i].pos * stm);

	if ((Tempip != NULL) && ((index%1000)==0))
		{
		Tempip->PopPrompt();
		}

//		}	


	}




Point3 GenParticle::GetSplineVelocity(int Index, TimeValue t)
{
float d = 0.0f;
float StartPer, PrevPer;
int life;
int UseAbsolute=1;
if  ((parts.ages[Index] == -1) || (parts.ages[Index] == 0) )
	return Point3(0.0f,0.0f,0.0f);



pblock->GetValue(PB_ABSOLUTE_DIRECTION,t,UseAbsolute,FOREVER);



Matrix3 ETM,STM;

ETM = EmitterTM;//ObjectNode->GetObjTMAfterWSM(t);

STM = SplineTM;//SplineCageNode->GetObjTMAfterWSM(t);




int CActive,CFrame;
int stepSize;

stepSize = GetTicksPerFrame();
pblock->GetValue(PB_CUTTERFRAME,t,CFrame,FOREVER);

pblock->GetValue(PB_CUTTERACTIVE,t,CActive,FOREVER);

pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);

if (TestAFlag(A_RENDER)) 
   life = life*2;





if  (!CActive)

	{

	StartPer = (float)(parts.ages[Index]+1)/(float)(life-1);
	PrevPer = (float)(parts.ages[Index])/(float)(life-1);
	
	if ((StartPer > 1.0f) || (PrevPer < 0.0f))
		{
		int fp;
		fp = 4;
		}


	}
else 
	{
	if (Starting[Index%NumberStrands].cutted)
		{
		float l = 0.0f;
		float p;


		for (int i=0;i<NumberOfSplines;i++)
			{
//			p = (Inversed[i]/d);
			p =  Starting[Index%NumberStrands].PercentControl[i];
 		    l += SplineLengths[i] * p;
			}
		StartPer = (float)(parts.ages[Index]+1) * (Starting[Index%NumberStrands].StrandLength/(float)life)/l;
		PrevPer = (float)(parts.ages[Index]) * (Starting[Index%NumberStrands].StrandLength/(float)life)/l;
		StartPer = (float)(parts.ages[Index]+1)/(float)life;
		PrevPer = (float)(parts.ages[Index])/(float)life;
		}
	else
		{
		StartPer = (float)(parts.ages[Index]+1)/(float)life;
		PrevPer = (float)(parts.ages[Index])/(float)life;
		}
	}




//put in here checks for Cutters active and then used % of spline used

//get vector for emmitter vertex;
Point3 EmitterVector,SplineVector,Perp,FirstPoint,Direction;
Point3 TDirection;
float angle;
Matrix3 Rotate;

Point3 PrevPoint,CurrentPoint;
PrevPoint.x = 0.0f;
PrevPoint.y = 0.0f;
PrevPoint.z = 0.0f;
CurrentPoint.x = 0.0f;
CurrentPoint.y = 0.0f;
CurrentPoint.z = 0.0f;

Direction.x = 0.0f;
Direction.y = 0.0f;
Direction.z = 0.0f;

TDirection.x = 0.0f;
TDirection.y = 0.0f;
TDirection.z = 0.0f;





//EmitterVector = Starting[Index%NumberStrands].vel;
//EmitterVector = VectorTransform(ETM,EmitterVector);
//EmitterVector = Normalize(EmitterVector);
//transform to world space

EmitterVector = StartingVelTransFormed[Index%NumberStrands];

if  (!CActive)
	Starting[Index%NumberStrands].StrandLength = 0.0f;
else 
	{
	if (Starting[Index%NumberStrands].cutted)
		{

//		if  ((!CActive)  || (t < (CFrame*stepSize)) )
		}
	else
		Starting[Index%NumberStrands].StrandLength = 0.0f;
	}




for (int i=0;i<NumberOfSplines;i++)
	{


//get spline vector
//	SplineVector = SplineStartingVerts[i];
//transform to world space
//	SplineVector = VectorTransform(ETM,SplineVector);
//get angle between emmitter vector and spline vector
//	SplineVector = Normalize(SplineVector);
	SplineVector = SplineStartingVertsTransformed[i];

	if ((SplineVector == EmitterVector) || (UseAbsolute))
		{
//		Rotate.IdentityMatrix();
		}
	else
		{

		angle = (float) acos(DotProd(EmitterVector,SplineVector));

//get perpendicular
		Perp = CrossProd(EmitterVector,SplineVector);


//trasnfer to matrix3
		Rotate.IdentityMatrix();
		Rotate=RotAngleAxisMatrix(Perp, -angle);
		}

//rotated spline start and end
	CurrentPoint = SplineVelocityList[i].Velocities[parts.ages[Index]];

	if (!((SplineVector == EmitterVector) || (UseAbsolute)))
		CurrentPoint = CurrentPoint * Rotate;

//set CurrentPoint to Pre computed data

	TDirection = Normalize(CurrentPoint);
if (TDirection.x == 1.0f)
	{
	DebugPrint("ErroR %d\n",parts.ages[Index]);
	}


	float p;

	if (NumberOfSplines > 1)
		p =  Starting[Index%NumberStrands].PercentControl[i];
	else p = 1.0f;

//DebugPrint("Strand %d Spline %d Percent Control %0.2f\n",Index%NumberStrands,i,p);

	TDirection *= p;


//get cutter frame 

	 if  (!CActive)
 		Starting[Index%NumberStrands].StrandLength += SplineLengths[i] * p;

	 else 
		{
		if (Starting[Index%NumberStrands].cutted)
			{
//		if  ((!CActive)  || (t < (CFrame*stepSize)) )
			}
		else
 		Starting[Index%NumberStrands].StrandLength += SplineLengths[i] * p;
		}
	

	Direction += TDirection;
	}

//DebugPrint("  SplineLength %4.2f\n",Starting[Index%NumberStrands].StrandLength);

if (Direction.x == 1.0f)
	{
	DebugPrint("ErroR\n");
	}
return Direction;



}


void GenParticle::DoEmitterCollision()
{

}

void GenParticle::DoNormalization(int CActive, int CFrame, TimeValue life, TimeValue dt)
{
	int count = CountLive();
	int CurrentParticle,PreviousParticle;
	float pdist,tdist;
	Point3 CP,PP;


	for (int l=0; l<NumberStrands; l++) {

		
//	DebugPrint("Normalizing strand %d\n",l);

		CurrentParticle = LastParticle-l;

		PreviousParticle = CurrentParticle;
		CurrentParticle -= NumberStrands;
		if (CurrentParticle<0)
			CurrentParticle =  count+CurrentParticle;


		for (int j=1;j<count/NumberStrands;j++)
			{




//check here for length tp previous particle
//get distance between CurrentPartocle and LastParticle
			if (!CActive)
				tdist = Starting[CurrentParticle%NumberStrands].StrandLength/life * 
					GetLengthPixel(Starting[CurrentParticle%NumberStrands].uvw.x,Starting[CurrentParticle%NumberStrands].uvw.y);
			else if ((CActive) && (dt <= (CFrame*stepSize)))
				tdist = Starting[CurrentParticle%NumberStrands].StrandLength/life * 
					GetLengthPixel(Starting[CurrentParticle%NumberStrands].uvw.x,Starting[CurrentParticle%NumberStrands].uvw.y);
			else
				tdist = Starting[CurrentParticle%NumberStrands].StrandLength/life; 

			pdist = Length(parts[CurrentParticle]-parts[PreviousParticle]);
			if ( tdist != pdist )
				{
//get normal vector
				Point3 DVec;
				CP = parts[CurrentParticle];
				PP = parts[PreviousParticle];
//DebugPrint("C %d P %d CP %f %f %f PP %f %f %f\n",CurrentParticle,PreviousParticle,CP.x,CP.y,CP.z,PP.x,PP.y,PP.z);
				DVec = parts[CurrentParticle]-parts[PreviousParticle];

				if (Length(DVec) > 0.0f)
					{
					DVec = 	Normalize(DVec) ;
					DVec *= tdist;
					parts[CurrentParticle] = parts[PreviousParticle] + DVec;
					}

				}
			PreviousParticle = CurrentParticle;
			CurrentParticle -= NumberStrands;
			if (CurrentParticle<0)
				CurrentParticle =  count+CurrentParticle;
			}
		}



}


void GenParticle::DoClumping(TimeValue life, float MaxLength, float Falloff)
{

	
int count = CountLive();
int CurrentParticle;
	
for (int l=0; l<NumberStrands; l++) {
	CurrentParticle = LastParticle-l;
			
//	CurrentParticle -= NumberStrands;
	if (CurrentParticle<0)
		CurrentParticle =  count+CurrentParticle;

	int strand = CurrentParticle%NumberStrands;
//check of not part of group
	if (count >0)
		{
		if ((Starting[strand].Group >= 0 ) && (Starting[strand].Group != strand))
			{
			for (int j=0;j<count/NumberStrands;j++)
				{
//find center and move particle % to to center;
				Point3 ParticleCenter,Vec;	

 				float falloff;
				float templife;
				templife = (float) parts.ages[CurrentParticle];
				if (templife <0.0f) templife = 0.0f;
				falloff = templife/(float)life;

				falloff =  ((templife) * Starting[j%NumberStrands].StrandLength/life)/MaxLength;
		
				if (falloff <0.0f ) falloff = 0.0f;
				if (falloff >1.0f ) falloff = 1.0f;

				if (falloff<Falloff)
					falloff = (Falloff-falloff)/Falloff;
					else falloff = 0.0f;

				if (ClumpingGraphNode != NULL)
					{
					falloff = GetGraph(CLUMPINGOBJECT_REF,tvalid,falloff);
					}




				int c = CurrentParticle - (strand - Starting[strand].Group);
				ParticleCenter = parts[c];
				Vec = ParticleCenter - parts[CurrentParticle];
				parts[CurrentParticle] = parts[CurrentParticle] + (Vec * falloff);


//DebugPrint("Part %d Strand %d Group %d Center Part %d\n",CurrentParticle,strand,Starting[strand].Group,c);
				CurrentParticle -= NumberStrands;
				if (CurrentParticle<0)
					CurrentParticle =  count+CurrentParticle;
				}					  
			}
					
		}
	}

}


void GenParticle::UpdateParticles(TimeValue t,INode *node)
	{
	TimeValue t0, life, dt;
	int j, constant = TRUE, total, birth;
	float brate, brateFactor = 1.0f;
	Point3 force;		
	float Falloff;
	float SLength;
//	int SampleRate;
	int secondary,MotionTransfer;
	int VertexDeformation;
	float initVel;
	int CFrame,CActive;
	Matrix3 tm;// = node->GetObjTMBeforeWSM(bt);
	float MaxLength;
	
	// The size of steps we take to integrate will be 
	// frame size steps. Unless we're rendering in which
	// case we'll take field size steps to ensure that
	// particle are synchronized on multiple machines
	// rendering fields


	int Go;
	if (TestAFlag(A_RENDER))
		{

		if (stepSize!=GetTicksPerFrame()/2) {
			stepSize = GetTicksPerFrame()/2;
 			valid = FALSE;
			}

	     pblock->GetValue(PB_RENDERACTIVE,t,Go,FOREVER);
		 if (!Go) 
			{
			for (j=0; j<parts.Count(); j++) 
				parts.ages[j] = -1;
			return;
			}
		}
	else
		{
		if (stepSize!=GetTicksPerFrame()) {
			stepSize = GetTicksPerFrame();
			valid = FALSE;
			}
	     pblock->GetValue(PB_VIEWACTIVE,t,Go,FOREVER);
		 if (!Go) 
			{
			for (j=0; j<parts.Count(); j++) 
				parts.ages[j] = -1;
			return;
			}
		}


	pblock->GetValue(PB_STARTTIME,t,t0,FOREVER);
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);
	if (TestAFlag(A_RENDER)) 
		life = life *2;

	pblock->GetValue(PB_DROPSIZE,t,parts.size,FOREVER);	
	pblock->GetValue(PB_WATJE_FALLOFF,t,Falloff,FOREVER);	

	pblock->GetValue(PB_MOTIONTRANSFER,t,MotionTransfer,FOREVER);
	pblock->GetValue(PB_VERTEXDEFORMATION,t,VertexDeformation,FOREVER);

	pblock->GetValue(PB_SPEED,t,initVel,FOREVER);

	int WSM_Modified;

	pblock->GetValue(PB_WSM,t,WSM_Modified,FOREVER);
	pblock->GetValue(PB_CUTTERFRAME,t,CFrame,FOREVER);
	pblock->GetValue(PB_CUTTERACTIVE,t,CActive,FOREVER);

	pblock->GetValue(PB_SETTLE,t,Settle,FOREVER);
	pblock->GetValue(PB_WAVES,t,Waves,FOREVER);


	if (SplineCageNode == NULL)
		pblock->GetValue(PB_LENGTH_MAX,t,MaxLength,FOREVER);
		else
		{
		SplineShape *SplineCage;

		ObjectState os;
		os = SplineCageNode->EvalWorldState(t);
		SplineCage = (SplineShape*)os.obj;


		Spline3D *spline = SplineCage->shape.GetSpline(0);
		MaxLength = spline->SplineLength();

		for (int i=1;i<SplineCage->shape.SplineCount();i++)
			{
			spline = SplineCage->shape.GetSpline(i);
			if (spline->SplineLength() > MaxLength)
				MaxLength = spline->SplineLength();
			}

		}




	SetLengthImageTime(t);
	


	total = 0xFFAA;

	total = GetStrandCount();

	if (life <= 0) life = 1;

	brate = float(total);		
	NumberStrands = total;
	pblock->GetValue(PB_SECONDARY,t,secondary,FOREVER);

	if ((secondary) && (SecondaryMotion.Count() != (total*life)))
		{
		SecondaryMotion.ZeroCount();
		SecondaryMotion.SetCount((total*life));
		SwingTime = 0;
		JiggleTime = 0;
		UseJiggle = 0;
		Affect = 0.0f;
		}
	
//DebugPrint("number strands at udpdate %d\n",NumberStrands);
 	if (t < t0) {
		// Before the start time, nothing is happening
		parts.FreeAll();
		tvalid = t;
		valid  = TRUE;
		return;
		}

	if (ObjectNode == NULL)
		return;

	if (!WSM_Modified)
		{
		if (!valid || t<tvalid || tvalid<t0) {
		// Set the particle system to the initial condition
			ComputeParticleStart(t0,node);
			}
		}
		else
		{
		int c = GetParticleCount();	
		if ((c!=parts.Count()) || t<tvalid || tvalid<t0) {
		// Set the particle system to the initial condition
			ComputeParticleStart(t0,node);
			}
		}

	valid = TRUE;

	
	char Prompt[80];



	if (tvalid==t0)
		{
		UseJiggle = 0;
		LastParticle = 0;
		tvalid = t0 - (life*stepSize);
		}

	if ((t-tvalid) > (life*stepSize))
		tvalid = t - (life*stepSize);
	while (tvalid < t) {
		int born = 0;		


		if (Tempip != NULL)
			{
			sprintf(Prompt,"Updating Particles %d",tvalid);
			Tempip->PushPrompt(_T(Prompt));
			}

		// Compute our step size
		if (tvalid%stepSize !=0) {
			dt = stepSize - tvalid%stepSize;
		} else {
			dt = stepSize;
			}
		if (tvalid + dt > t) {
			dt = t-tvalid;
			}

		// Increment time
		tvalid += dt;

		
		// Compute the number of particles that should be born
//		if (!constant) {
//			pblock->GetValue(PB_BIRTHRATE,tvalid,brate,FOREVER);
//			}
		birth = (int((tvalid-t0)*brate*brateFactor) 
			- int((tvalid-t0-dt)*brate*brateFactor))/stepSize;
				

// First increment age and kill off old particles
//	DebugPrint("about to kill\n");



//do secondary motion


		// We want to seed the random number generator for all particles
		// born at this time. Construct the seed based on this time by
		// pulling out each of the 4 bytes, Perm() each one, and put
		// them back in the opposite byte order.
		int seed1 = (tvalid*1200)/TIME_TICKSPERSEC;
		int seed2 = seed1 >> 8;
		int seed3 = seed2 >> 8;
		int seed4 = seed2 >> 8;
		seed1 &= 0xFF;
		seed2 &= 0xFF;
		seed3 &= 0xFF;
		seed4 &= 0xFF;
		srand(
			(Perm(seed1)<<24) + 
			(Perm(seed2)<<16) +
			(Perm(seed3)<<8)  +
			Perm(seed4) +
			int(PARTICLE_SEED));

		PrevEmitterTM = ObjectNode->GetObjTMAfterWSM(t-stepSize);
		EmitterTM=ObjectNode->GetObjTMAfterWSM(t);
		if (SplineCageNode != NULL) SplineTM=SplineCageNode->GetObjTMAfterWSM(t);

//DebugPrint("Frame %d \n",t/160);
//check is need to do settling frames
		if (secondary)
			{


//DebugPrint(" Jiggle %d\n",UseJiggle);
			if (((tvalid-SwingTime)>(Settle*160)) || (tvalid < JiggleTime))
				{
				UseJiggle = 0;
				}


			Point3 P(1.0f,1.0f,1.0f),C(1.0f,1.0f,1.0f);
			P = P * PrevEmitterTM;
			C = C * EmitterTM;


			PrevStill = Still;
			float dist = Length(P-C);
			if (dist<0.0001) //&& (UseJiggle == 0)) 
				Still = TRUE;
				else Still = FALSE;


			if (UseJiggle >= 1)
				{
				if (!Still)
					UseJiggle = 0;	
				else
					UseJiggle++;
				}
			if ((Still) && (!PrevStill))
				{	
				SwingTime = tvalid;
//			if (UseJiggle == 1)
//				{
		 		JiggleTime = tvalid;
				UseJiggle = 1;
				}


			if (UseJiggle > 1)
				{
				float s =(float) (Settle*160);
				float jt = (float)(tvalid-JiggleTime);
//DebugPrint("Settle %f jt %f Waves %d  tvalid %d JiggleTime %d dist %f\n",s,jt,Waves,tvalid,JiggleTime,dist);
				
				Affect = (float) cos(
							  (jt/s * Waves* 3.14159265359f)) * 

							  (s - jt) / s 	 ;
				}
			else
				{
				Affect = 0.0f;
				}

//DebugPrint("Frame %d Affect is %f\n",tvalid/stepSize,Affect)	;
			}



/*
		if ((secondary) && (UseJiggle ==0) )
			{
			int c = 0;
			int CurrentParticle;
			int count = CountLive();
			for (int l=0; l<NumberStrands; l++) 
				{
				CurrentParticle = LastParticle-l;

DebugPrint("Strand %d\n",l);
				for (int j=0;j<count/NumberStrands;j++)
					{

					SecondaryMotion[c++] = parts[CurrentParticle];


DebugPrint("    Part %d Pos %f %f %f\n",j,parts[CurrentParticle].x,parts[CurrentParticle].y,parts[CurrentParticle].z);
		
					CurrentParticle -= NumberStrands;
					if (CurrentParticle<0)
						CurrentParticle =  count+CurrentParticle;
					}
				}
			
			}

  */


		if (Tempip != NULL)
			{
			sprintf(Prompt,"Building Particle Starting Pos %d",tvalid);
			Tempip->PushPrompt(_T(Prompt));
			}

		if ((ObjectNode != NULL) && ((Starting.Count() == 0) ||(Starting.Count() != NumberStrands)))
			{
//			DebugPrint("about to rebuild starting pos\n");
			BuildStartingPos(TRUE,tvalid);
//			BuildVerts();
//			Starting.ZeroCount();
			}
//check if ObjectNode has changed fix this
		else{
			BuildStartingPos(FALSE,tvalid);
			}


		
		if (Tempip != NULL)
			{
			Tempip->PopPrompt();
			}

		Point3 TempVel,InitVel;

		// Apply forces to modify velocity
		if (Tempip != NULL)
			{
			int BID = BETAID;
			sprintf(Prompt,"Applying Forces %d %d",tvalid,BID);
			Tempip->PushPrompt(_T(Prompt));
			}


		tm = ObjectNode->GetObjTMAfterWSM(tvalid);
//		tm = ObjectNode->GetObjectTM(tvalid);

	// Transform the point and velocity into our space
//		tm = ObjectNode->GetObjTMBeforeWSM(bt);
//		tm    = node->GetObjectTM(t,&tmValid);

		Matrix3 mtm,invtm,stm;// = node->GetObjTMBeforeWSM(bt);
		Interval tmValid;		
		Point3 PrevPos,NextPos;
//		mtm    = ObjectNode->GetObjectTM(tvalid-stepSize,&tmValid);
		mtm    = ObjectNode->GetObjTMAfterWSM(tvalid-stepSize);
		invtm = Inverse(mtm);


		for (j=0; j<parts.Count(); j++) {
			float falloff;
			float templife;
			templife = (float) parts.ages[j];
			if (templife <0.0f) templife = 0.0f;


//			falloff = templife/(float)life;
//check for length age * length/Max
			falloff =  ((templife) * Starting[j%NumberStrands].StrandLength/life)/MaxLength;

			if (falloff <0.0f ) falloff = 0.0f;
			if (falloff >1.0f ) falloff = 1.0f;

			if (falloff<Falloff)
				falloff = (Falloff-falloff)/Falloff;
				else falloff = 0.0f;


			TempVel.x = 0.0f;
			TempVel.y = 0.0f;
			TempVel.z = 0.0f;


			
			for (int i=0; i<fields.Count(); i++) {		
				if (!parts.Alive(j)) continue;
				
				force = fields[i]->Force(tvalid,parts[j],parts.vels[j],j);
//				parts.vels[j] += force * float(dt);
				TempVel += force * float(dt);
				}

			Point3 dir,dir2;

			if (fields.Count() != 0)
		 		dir = (TempVel);
//		 		dir = Normalize(TempVel);
				else dir = TempVel;

			if (SplineCageNode == NULL)
				{
				dir2 = (VectorTransform(tm,Starting[j%NumberStrands].vel)*initVel);
				dir2 = dir2 * falloff;
				}
			else
				{	
  				dir2 = GetSplineVelocity(j,tvalid);
				}
//			dir2 = Normalize(VectorTransform(tm,Starting[j%NumberStrands].vel)*initVel);

			dir = dir * (1.0f-falloff);

//			dir = ((TempVel*(1.0f-falloff))+(parts.vels[j]*falloff));
  			dir = dir+dir2;
			if (Length(dir) > 0.01f)
				dir = Normalize(dir);
			if (!CActive)
				SLength = Starting[j%NumberStrands].StrandLength/life * GetLengthPixel(Starting[j%NumberStrands].uvw.x,Starting[j%NumberStrands].uvw.y);
			else if ((CActive) && (tvalid <= (CFrame*stepSize)))
				SLength = Starting[j%NumberStrands].StrandLength/life * GetLengthPixel(Starting[j%NumberStrands].uvw.x,Starting[j%NumberStrands].uvw.y);
			else
				SLength = Starting[j%NumberStrands].StrandLength/life;
			dir = dir *SLength;


//change this to momentum

			if (secondary)
				{
/*
				Point3 P(1.0f,1.0f,1.0f),C(1.0f,1.0f,1.0f);
				P = P * PrevEmitterTM;
				C = C * EmitterTM;

				PrevStill = Still;
				if (Length(P-C)<0.00001) //&& (UseJiggle == 0)) 
					Still = TRUE;
					else Still = FALSE;
				if ((!Still) && (PrevStill))
					SwingTime = t;



				if (SecondaryGraphNode == NULL)
					dir +=(SecondaryMotion[j] * falloff);
					else
					{
					float p;
					p = GetGraph(SECONDARYOBJECT_REF,tvalid,falloff);
					dir += SecondaryMotion[j] * p;
					}
*/
				}


//apply Vertex Deformation
			if (VertexDeformation)
				{	
//put into local coord - stepsize
				NextPos = Starting[j%NumberStrands].pos - Starting[j%NumberStrands].prevpos; 


//do rotation	get prev normal & new normal

				if ((Starting[j%NumberStrands].vel !=  Starting[j%NumberStrands].prevnormal) && (Length(Starting[j%NumberStrands].prevnormal) != 0.0f))
					{
					Matrix3 Rotate;
					float angle;
					Point3 Perp;
					Rotate.IdentityMatrix();

					angle = (float) acos(DotProd(Starting[j%NumberStrands].vel,
						                         Starting[j%NumberStrands].prevnormal));

//get perpendicular
					if (angle != 0.0f)
						{
						Perp = CrossProd(Starting[j%NumberStrands].vel,
						                     Starting[j%NumberStrands].prevnormal);


//trasnfer to matrix3
//	QRotate.MakeMatrix(Rotate);
//					Rotate.IdentityMatrix();
						Rotate=RotAngleAxisMatrix(Perp, -angle);
						Point3 Temp,TempPrev;
						Temp =  parts[j] - Starting[j%NumberStrands].pos; 
						TempPrev = Temp;
						Temp = Temp * Rotate;
						Temp = Temp-TempPrev;
						NextPos += Temp;
						}
					}


				NextPos = VectorTransform(tm,NextPos);
				if (VertexDeformationGraphNode != NULL)
					{
					float p;
					p = GetGraph(VERTEXDEFORMATIONOBJECT_REF,tvalid,falloff);
					NextPos *=  p;
					}
				dir += NextPos;




//transform  to current time
				}

//apply motion Transder
			if (MotionTransfer)
				{	
//put into local coord - stepsize
				PrevPos = parts[j];
				NextPos = PrevPos * invtm; 
				NextPos = NextPos * tm;
				NextPos -= PrevPos;
				if (MotionGraphNode == NULL)
					NextPos *=  falloff;
					else {
					float p;
 					p = GetGraph(MOTIONOBJECT_REF,tvalid,falloff);
					NextPos *=  p;
					}
				dir += NextPos;
//transform  to current time
				}






			parts.vels[j] = dir;
 
			// Check for collisions
			BOOL collide = FALSE;
			

//			if (((CActive) && (tvalid > (CFrame*stepSize)))  || (!CActive))
			if (!CActive)
				{
				for (i=0; i<cobjs.Count(); i++) {
					if (cobjs[i]->CheckCollision(
							tvalid,parts[j],parts.vels[j], 1.0f/*float(dt)*/ ,j)) 
						{
						collide = TRUE;
							
//set length for this strand
//new length = age * seg length + distance from last particle to this particle
							

						}
					}

				}
			
			// If we didn't collide, then increment.

			if ( (cobjs.Count()==0) || ( (collide==FALSE)) )
				parts[j] +=  dir;// * float(dt);			
	//			parts[j] += parts.vels[j] * float(dt);			

			}

		if (Tempip != NULL)
			{
			Tempip->PopPrompt();
			}
	
		// Next, birth particles at the birth rate
//	DebugPrint("	about to birth\n");

		if (secondary)
//		stm = ObjectNode->GetObjTMBeforeWSM(bt-TPerFrame);
			stm = ObjectNode->GetObjTMAfterWSM(tvalid-stepSize);


//check here for particles that stray to far or close
		int count = CountLive();
		int CurrentParticle,PreviousParticle;
//	float pdist,tdist;
		Point3 CP,PP;
//DebugPrint("Start normalizong\n");



//clumping code here only do clumping if rendering



		if (Tempip != NULL)
			{

			sprintf(Prompt,"Normalizing Strands %d",tvalid);
			Tempip->PushPrompt(_T(Prompt));
			}

//apply momentum
		if ((secondary) )
			{
			if (UseJiggle > 1)
//			if (UseJiggle >= 1000)
				{
				int ct = 0;
				for (int l=0; l<NumberStrands; l++) 
					{
					CurrentParticle = LastParticle-l;
				
//					PreviousParticle = CurrentParticle;
//					CurrentParticle -= NumberStrands;
					if (CurrentParticle<0)
						CurrentParticle =  count+CurrentParticle;

					for (int j=0;j<count/NumberStrands;j++)
						{


						float falloff;
						float templife;
						Point3 pt(0.0f,0.0f,0.0f);

						templife = (float) parts.ages[CurrentParticle];
						if (templife <0.0f) templife = 0.0f;
						falloff = templife/(float)life;

						falloff =  ((templife) * Starting[j%NumberStrands].StrandLength/life)/MaxLength;

						if (falloff <0.0f ) falloff = 0.0f;
						if (falloff >1.0f ) falloff = 1.0f;

						if (falloff<Falloff)
							falloff = (Falloff-falloff)/Falloff;
							else falloff = 0.0f;
						
						if (SecondaryGraphNode == NULL)

							pt +=(SecondaryMotion[ct++] * falloff * Affect);
							else
							{
							float p;
							p = GetGraph(SECONDARYOBJECT_REF,tvalid,falloff);
							pt += SecondaryMotion[ct++] * p * Affect;
							}
						parts[CurrentParticle] += pt;
						CurrentParticle -= NumberStrands;
						if (CurrentParticle<0)
							CurrentParticle =  count+CurrentParticle;


						}
					}
				}
			}

//do clumping
		if (TestAFlag(A_RENDER))
			{
//			DoClumping(life,MaxLength,Falloff);//CActive, CFrame, life, dt);
			}

//do emitter collision

//do normalizing

		if (1)//((tvalid%160) == 0)
			{
			if (Tempip != NULL)
				{
				sprintf(Prompt,"Killing Old Particles %d",tvalid);
				Tempip->PushPrompt(_T(Prompt));
				}

			for (j=0; j<parts.Count(); j++) {
				if (!parts.Alive(j)) continue;
				parts.ages[j] ++;
				if (parts.ages[j] >= life) {
					parts.ages[j] = -1;
					}
				}

			if (Tempip != NULL)
				{
				Tempip->PopPrompt();
				}


			if (Tempip != NULL)
				{

				sprintf(Prompt,"Birthing Particles %d",tvalid);
				Tempip->PushPrompt(_T(Prompt));
				}



		 	for (j=0; j<parts.Count(); j++) {
				if (born>=birth) break;
				if (!parts.Alive(j)) {
					BirthParticle(node,tvalid,j,stm);
					born++;
					}
				}
			if (Tempip != NULL)
				{
				Tempip->PopPrompt();
				}
		}




		DoNormalization(CActive, CFrame, life, dt);

	


//do cutters
		if ((CActive) && (tvalid <= (CFrame*stepSize)))
			{
//cycle through particle looking for intersections
			for (int l=0; l<NumberStrands; l++) 
				{
				CurrentParticle = LastParticle-l;

				PreviousParticle = CurrentParticle;
				CurrentParticle -= NumberStrands;
				if (CurrentParticle<0)
					CurrentParticle =  count+CurrentParticle;

				for (int j=1;j<count/NumberStrands;j++)
					{




//check here for length tp previous particle
//get distance between CurrentPartocle and LastParticle

//get normal vector
					Point3 DVec,T;
					CP = parts[CurrentParticle];
					PP = parts[PreviousParticle];
					T = CP;
					DVec = CP-PP;
					int collide;
					Starting[CurrentParticle%NumberStrands].cutted = FALSE;

					for (int i=0; i<cobjs.Count(); i++) 
						{
						if (cobjs[i]->CheckCollision(
							tvalid,PP,DVec,float(1.0f),j )) 
							{
//set new length age*setlength + diff;
							float vlen = Length(T-PP);
							float pl,cl;
							pl = Starting[CurrentParticle%NumberStrands].StrandLength;
							Starting[CurrentParticle%NumberStrands].StrandLength = 
								parts.ages[CurrentParticle] * 
								Starting[CurrentParticle%NumberStrands].StrandLength/life + vlen;
							Starting[CurrentParticle%NumberStrands].cutted = TRUE;
							cl = Starting[CurrentParticle%NumberStrands].StrandLength;
//							i = cobjs.Count()+1;
							j = (count/NumberStrands) +1;
							}
						}
					PreviousParticle = CurrentParticle;
					CurrentParticle -= NumberStrands;
					if (CurrentParticle<0)
						CurrentParticle =  count+CurrentParticle;
					}
				}


			}


//do secondary motion
		if ((secondary) && (UseJiggle == 0) )
//		if ((secondary) )
			{
			int c = 0;
//cycle through particle looking for intersections

			for (int l=0; l<NumberStrands; l++) 
				{
				CurrentParticle = LastParticle-l;

//				CurrentParticle -= NumberStrands;
//DebugPrint("After Move Strand %d\n",l);
//				if (CurrentParticle<0)
//					CurrentParticle =  count+CurrentParticle;
				
				for (int j=0;j<count/NumberStrands;j++)
					{

					SecondaryMotion[c++] = parts[CurrentParticle];

//DebugPrint("    Part %d Age %d Pos %f %f %f\n",j,parts.ages[CurrentParticle],parts[CurrentParticle].x,parts[CurrentParticle].y,parts[CurrentParticle].z);


					CurrentParticle -= NumberStrands;
					if (CurrentParticle<0)
						CurrentParticle =  count+CurrentParticle;
					}
				}

			}

		if ((secondary) && (UseJiggle == 1) )
//		if ((secondary) )
			{
			int c = 0;
//cycle through particle looking for intersections

			for (int l=0; l<NumberStrands; l++) 
				{
				CurrentParticle = LastParticle-l;

//				CurrentParticle -= NumberStrands;
//DebugPrint("After Move Strand %d\n",l);
//				if (CurrentParticle<0)
//					CurrentParticle =  count+CurrentParticle;
				
				for (int j=0;j<count/NumberStrands;j++)
					{

					SecondaryMotion[c] = parts[CurrentParticle]-SecondaryMotion[c++];

//DebugPrint("    Part %d Age %d Pos %f %f %f\n",j,parts.ages[CurrentParticle],parts[CurrentParticle].x,parts[CurrentParticle].y,parts[CurrentParticle].z);


					CurrentParticle -= NumberStrands;
					if (CurrentParticle<0)
						CurrentParticle =  count+CurrentParticle;
					}
				}

			}







/*
DebugPrint("Last Particle %d\n",LastParticle);
		for (int j=0;j<=count;j++)
			{


DebugPrint("    Part %d Age %d Pos %f %f %f\n",j,parts.ages[j],parts[j].x,parts[j].y,parts[j].z);

			}
*/





		if (Tempip != NULL)
			{

			Tempip->PopPrompt();
			Tempip->PopPrompt();
			}


		}


	

}
