#include "prim.h"
#include "iparamm.h"
#include "Simpobj.h"
#include "texutil.h"
#include "istdplug.h"
#include "decomp.h"
#include "trig.h"
#include "bmmlib.h"
#include "splshape.h"
#include "simpspl.h"

#include "hair.h"




Mesh *GenParticle::GetFacingRenderMesh(
		TimeValue t, INode *inode, View& view, BOOL& needDelete)
	{
	Mesh *pm = new Mesh;	
	int ix=0, nx=0, count;
	float size, sz;
	Matrix3 tm = Inverse(inode->GetObjTMAfterWSM(t));
	Matrix3 cam = Inverse(view.worldToView);
	Point3 v, v0,v1, camV = cam.GetRow(3);	
	TimeValue life;

	pblock->GetValue(PB_DROPSIZE,t,size,FOREVER);
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);
	if (TestAFlag(A_RENDER)) 
		life = life *2;

	if (life<=0) life = 1;

	Update(t,inode);
	count = CountLive();

	pm->setNumFaces(count*2);
	pm->setNumVerts(count*4);
	pm->setNumTVerts(count*4);
	pm->setNumTVFaces(count*2);

	for (int i=0; i<parts.Count(); i++) {
		if (!parts.Alive(i)) continue;

		// Compute this particle's size
		sz = CompParticleSize(t,parts.ages[i],life,size);
	 
		// Compute a vector from the particle to the camera
		v  = Normalize(camV-parts[i]);		
		v0 = Normalize(Point3(0,0,1)^v) * sz;
		v1 = Normalize(v0^v) * sz;
		
		pm->verts[ix  ] = (parts[i]+v0+v1) * tm;
		pm->verts[ix+1] = (parts[i]-v0+v1) * tm;
		pm->verts[ix+2] = (parts[i]-v0-v1) * tm;
		pm->verts[ix+3] = (parts[i]+v0-v1) * tm;

		pm->faces[nx  ].setSmGroup(1);		
		pm->faces[nx  ].setVerts(ix+1,ix ,ix+2);
		pm->faces[nx  ].setMatID((MtlID)i);
		pm->faces[nx+1].setSmGroup(1);		
		pm->faces[nx+1].setVerts(ix+3,ix+2,ix  );
		pm->faces[nx+1].setMatID((MtlID)i);

		pm->setTVert(ix  ,1.0f,1.0f,0.0f);
		pm->setTVert(ix+1,0.0f,1.0f,0.0f);
		pm->setTVert(ix+2,0.0f,0.0f,0.0f);
		pm->setTVert(ix+3,1.0f,0.0f,0.0f);

		pm->tvFace[nx  ].setTVerts(ix+1,ix  ,ix+2);
		pm->tvFace[nx+1].setTVerts(ix+3,ix+2,ix  );
		
		ix += 4;
		nx += 2;
		}
	
	mesh.InvalidateGeomCache();
	needDelete = TRUE;
	return pm;
	}






Mesh *SnowParticle::GetPlusMesh(
		TimeValue t, INode *inode, View& view, BOOL& needDelete)
	{
	Mesh *pm = new Mesh;	
	int ix=0, nx=0, count;
	float size, sz;
	float tumble, scale;
	Matrix3 tm = Inverse(inode->GetObjTMAfterWSM(t));
	Matrix3 cam = Inverse(view.worldToView);
	Point3 v, v0,v1, camV = cam.GetRow(3);	
	TimeValue life;

	pblock->GetValue(PB_DROPSIZE,t,size,FOREVER);
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);
	if (TestAFlag(A_RENDER)) 
		life = life *2;


	pblock->GetValue(PB_TUMBLE,t,tumble,FOREVER);
	pblock->GetValue(PB_SCALE,t,scale,FOREVER);


	if (life<=0) life = 1;

	Update(t,inode);
	count = CountLive();

	pm->setNumFaces(count*6);
	pm->setNumVerts(count*12);
	pm->setNumTVerts(count*12);
	pm->setNumTVFaces(count*6);

	for (int i=0; i<parts.Count(); i++) {
		if (!parts.Alive(i)) continue;

		// Compute this particle's size
		sz = CompParticleSize(t,parts.ages[i],life,size);

		pm->verts[ix+0].x = -sz/2.0f;
		pm->verts[ix+0].y = 0.0f;
		pm->verts[ix+0].z = sz/2.0f;

		pm->verts[ix+1].x = sz/2.0f;
		pm->verts[ix+1].y = 0.0f;
		pm->verts[ix+1].z = sz/2.0f;

		pm->verts[ix+2].x = sz/2.0f;
		pm->verts[ix+2].y = 0.0f;
		pm->verts[ix+2].z = -sz/2.0f;

		pm->verts[ix+3].x = -sz/2.0f;
		pm->verts[ix+3].y = 0.0f;
		pm->verts[ix+3].z = -sz/2.0f;


		pm->verts[ix+4].x = -sz/2.0f;
		pm->verts[ix+4].z = 0.0f;
		pm->verts[ix+4].y = sz/2.0f;

		pm->verts[ix+5].x = sz/2.0f;
		pm->verts[ix+5].z = 0.0f;
		pm->verts[ix+5].y = sz/2.0f;

		pm->verts[ix+6].x = sz/2.0f;
		pm->verts[ix+6].z = 0.0f;
		pm->verts[ix+6].y = -sz/2.0f;

		pm->verts[ix+7].x = -sz/2.0f;
		pm->verts[ix+7].z = 0.0f;
		pm->verts[ix+7].y = -sz/2.0f;


		
		pm->verts[ix+8].z = -sz/2.0f;
		pm->verts[ix+8].x = 0.0f;
		pm->verts[ix+8].y = sz/2.0f;

		pm->verts[ix+9].z = sz/2.0f;
		pm->verts[ix+9].x = 0.0f;
		pm->verts[ix+9].y = sz/2.0f;

		pm->verts[ix+10].z = sz/2.0f;
		pm->verts[ix+10].x = 0.0f;
		pm->verts[ix+10].y = -sz/2.0f;

		pm->verts[ix+11].z = -sz/2.0f;
		pm->verts[ix+11].x = 0.0f;
		pm->verts[ix+11].y = -sz/2.0f;



		pm->tVerts[ix+0].x = 0.0f;
		pm->tVerts[ix+0].y = 0.0f;
		pm->tVerts[ix+0].z = 0.0f;

		pm->tVerts[ix+1].x = 1.0f;
		pm->tVerts[ix+1].y = 0.0f;
		pm->tVerts[ix+1].z = 0.0f;

		pm->tVerts[ix+2].x = 1.0f;
		pm->tVerts[ix+2].y = 1.0f;
		pm->tVerts[ix+2].z = 0.0f;

		pm->tVerts[ix+3].x = 0.0f;
		pm->tVerts[ix+3].y = 1.0f;
		pm->tVerts[ix+3].z = 0.0f;

		pm->tVerts[ix+4].x = 0.0f;
		pm->tVerts[ix+4].y = 0.0f;
		pm->tVerts[ix+4].z = 0.0f;

		pm->tVerts[ix+5].x = 1.0f;
		pm->tVerts[ix+5].y = 0.0f;
		pm->tVerts[ix+5].z = 0.0f;

		pm->tVerts[ix+6].x = 1.0f;
		pm->tVerts[ix+6].y = 1.0f;
		pm->tVerts[ix+6].z = 0.0f;

		pm->tVerts[ix+7].x = 0.0f;
		pm->tVerts[ix+7].y = 1.0f;
		pm->tVerts[ix+7].z = 0.0f;

		pm->tVerts[ix+8].x = 0.0f;
		pm->tVerts[ix+8].y = 0.0f;
		pm->tVerts[ix+8].z = 0.0f;

		pm->tVerts[ix+9].x = 1.0f;
		pm->tVerts[ix+9].y = 0.0f;
		pm->tVerts[ix+9].z = 0.0f;

		pm->tVerts[ix+10].x = 1.0f;
		pm->tVerts[ix+10].y = 1.0f;
		pm->tVerts[ix+10].z = 0.0f;

		pm->tVerts[ix+11].x = 0.0f;
		pm->tVerts[ix+11].y = 1.0f;
		pm->tVerts[ix+11].z = 0.0f;




		for (int j=0; j<12; j++) {
			pm->verts[ix+j]  = (pm->verts[ix+j]*TumbleMat(i,tumble,scale)) * sz;
			pm->verts[ix+j]  += parts[i];
			
			// Particles are in world space so we transform them back
			// into object space because that's what the renderer expects.
			pm->verts[ix+j] = tm * pm->verts[ix+j];
			}
		
		for (j=0; j<6; j++) {
			pm->faces[nx+j].setSmGroup(0);
			pm->faces[nx+j].setMatID((MtlID)i);
			}

		pm->faces[nx].setVerts(ix,ix+1,ix+3);
		pm->tvFace[nx].setTVerts(ix,ix+1,ix+3);

		pm->faces[nx+1].setVerts(ix+1,ix+2,ix+3);
		pm->tvFace[nx+1].setTVerts(ix+1,ix+2,ix+3);

		pm->faces[nx+2].setVerts(ix+4,ix+5,ix+7);
		pm->tvFace[nx+2].setTVerts(ix+4,ix+5,ix+7);

		pm->faces[nx+3].setVerts(ix+5,ix+6,ix+7);
		pm->tvFace[nx+3].setTVerts(ix+5,ix+6,ix+7);

		pm->faces[nx+4].setVerts(ix+8,ix+9,ix+11);
		pm->tvFace[nx+4].setTVerts(ix+8,ix+9,ix+11);

		pm->faces[nx+5].setVerts(ix+9,ix+10,ix+11);
		pm->tvFace[nx+5].setTVerts(ix+9,ix+10,ix+11);


		ix += 12;
		nx += 6;
		}
	
	mesh.InvalidateGeomCache();
	needDelete = TRUE;
	return pm;
	}

	

	
//get number of active strands
int GenParticle::RemoveStrands(View& view,TimeValue t, Matrix3 tm,int& AdditionSegs, int& AdditionNumber)

{
	int UseAngleLimit,UseFillerAngleLimit;
	int Angle = 180;
	float fangle,ffangle;
	float nfangle,nffangle;
	int ct = 0,count=0;
	int SegNumber = 0;
	Matrix3 cam = Inverse(view.worldToView);
	Point3 Target(0.0f,0.0f,1.0f);
	Target = Target*cam;
	Point3 v,camV = cam.GetRow(3);	
//get view vector
	Point3 CameraVector = Normalize(Target-camV);

	pblock->GetValue(PB_USE_ANGLE_LIMIT,t,UseAngleLimit,FOREVER);	

	pblock->GetValue(PB_USE_FILLER_ANGLE_LIMIT,t,UseFillerAngleLimit,FOREVER);	

	if (!UseAngleLimit) 
		{
		for (int i=0;i<NumberStrands;i++)
			{
			Starting[i].on = TRUE;
			}
		}	

	if (!UseFillerAngleLimit) 
		{
		for (int i=0;i<NumberStrands;i++)
			{
			Starting[i].use_filler = TRUE;
			}
		}	
	
	pblock->GetValue(PB_ANGLE_LIMIT,t,Angle,FOREVER);	

//convert angle to radians;
	fangle = (float)Angle/180.0f*3.14f;
	nfangle = (float)fabs(fangle);
	
	pblock->GetValue(PB_FILLER_ANGLE_LIMIT,t,Angle,FOREVER);	

//convert angle to radians;
	ffangle = (float)Angle/180.0f*3.14f;
	nffangle = (float)fabs(ffangle);


//Nuke RemoveData

	FillerData.ZeroCount();

	FillerClass TempFiller;

	TempFiller.OffsetList.ZeroCount();
	TempFiller.SegmentList.ZeroCount();


//get field mult.
	int Mult,Min,Max,SegDist;
	float dist;
	pblock->GetValue(PB_FILLER_MULTIPLIER,t,Mult,FOREVER);	
	pblock->GetValue(PB_MIN_FILLER_SEGMENT,t,Min,FOREVER);	
	pblock->GetValue(PB_MAX_FILLER_SEGMENT,t,Max,FOREVER);	
	pblock->GetValue(PB_FILLER_DISTANCE,t,dist,FOREVER);	
	SegDist = Max-Min;




	for (int i=0;i<NumberStrands;i++)
		{
		FillerData.Append(1,&TempFiller,1);
		}




	for (i=0;i<NumberStrands;i++)
		{
//Get Launch Vector
		Point3 v1;
		ct = LastParticle-i;
//		v1  = Normalize((Starting[i].pos*tm)-camV);		

		v  = VectorTransform(tm,Normalize(Starting[i].vel));
		float VAngle;

		VAngle = (float) acos(DotProd(CameraVector,v));

//Compute Angle between camera and launch
		 
		if ((fangle > 0) && (VAngle > fangle) && (UseAngleLimit))
			{
			Starting[i].on = FALSE;
			Starting[i].use_filler = FALSE;
			}
		else if ((fangle < 0) && (VAngle < nfangle) && (UseAngleLimit))
			{
			Starting[i].on = FALSE;
			Starting[i].use_filler = FALSE;
			}
		else
			{
			Starting[i].on = TRUE;
			count++;
			if ((ffangle > 0.0f) && (VAngle > ffangle) && (UseFillerAngleLimit) )
				{
				Starting[i].use_filler = FALSE;
				}
			else if ((ffangle < 0.0f) && (VAngle < nffangle) && (UseFillerAngleLimit) )
				{
				Starting[i].use_filler = FALSE;
				}
			else
				{
				Point3 p;
				int SegLength;
				Starting[i].use_filler = TRUE;
//fill in filler data here
//set rand seed
				srand(int(Starting[i].seed));

//loop through multiplier


				for (int j = 0;j<Mult;j++)
					{
//get rand length
					SegLength = (Min+ (int) ((float)SegDist * (rand()/float(RAND_MAX))))*2;
					SegNumber += SegLength;
					AdditionNumber++;
					FillerData[i].SegmentList.Append(1,&SegLength,1);
//get rand pos
					p.x = dist * (rand()/float(RAND_MAX));
					p.y = dist * (rand()/float(RAND_MAX));
					p.z = dist * (rand()/float(RAND_MAX));
					FillerData[i].OffsetList.Append(1,&p,1);
					}

				}

			}



//mark as invisible


		}

	
	AdditionSegs = SegNumber;

	if (UseAngleLimit) 
		return count;
	else return NumberStrands;



}

// The ratio of the base of the tetrahedron to its height
#define BASEFACTOR	0.05f
#define THIRD 		(1.0f/3.0f)
#define SIXTH 		(1.0f/6.0f)
#define TWOTHIRD 	(2.0f/3.0f)
#define FIVESIXTHS 	(5.0f/6.0f)



Mesh *SnowParticle::GetHairMesh(
		TimeValue t, INode *inode, View& view, BOOL& needDelete)
	
	{
	Mesh *pm = new Mesh;
	Point3 v, v0, v1;
	int ix=0, nx=0, count, type;
	Matrix3 tm = Inverse(inode->GetObjTMAfterWSM(t));
	TimeValue life;
	float sz;

	char Prompt[80];
	int GenerateW;

	BOOL needDel;
	ObjectState os = ObjectNode->EvalWorldState(t);
	NullView nullView;

	assert(os.obj->SuperClassID()==GEOMOBJECT_CLASS_ID);

 	Mesh *EmitterMesh = ((GeomObject*)os.obj)->GetRenderMesh(t,ObjectNode,nullView,needDel);

	Face *FaceList = EmitterMesh->faces;



	pblock->GetValue(PB_RENDER,t,type,FOREVER);	
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);
	if (TestAFlag(A_RENDER)) 
		life = life *2;

	pblock->GetValue(PB_GENERATEW,t,GenerateW,FOREVER);
	if (life<=0) life = 1;

	Update(t,inode);

 	count = CountLive();

	if (count != (NumberStrands*life))
		{
		pm->setNumFaces(0);
		pm->setNumVerts(0);
		mesh.InvalidateGeomCache();
		needDelete = TRUE;
		return pm;
		}


	if (Tempip != NULL)
		{
		sprintf(Prompt,"Building Mesh %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}


	Matrix3 tm2 = ObjectNode->GetObjTMAfterWSM(tvalid);
	int ASegs = 0;
	int SegC  = 0;
	int NumberOn = RemoveStrands(view,t,tm2,ASegs,SegC);


//	count = Life * NumberOn;

//finish code here

//	pm->setNumFaces((count-NumberStrands)*6);
//	pm->setNumVerts(count*3);
	int totalface,totalvertex;

	totalface = ((NumberOn*life)-NumberOn)*6 + (6*(ASegs-SegC));
	totalvertex = (NumberOn*life*3)+ (3*(ASegs));

	pm->setNumFaces(((NumberOn*life)-NumberOn)*6 + (6*(ASegs-SegC)));
	pm->setNumVerts((NumberOn*life*3)+ (3*(ASegs)));


	if (Tempip != NULL)
		{

		Tempip->PopPrompt();
		}


//	DebugPrint("Number Strands %d, face %d verts %d\n",NumberStrands,(count-NumberStrands)*6,count*3);

//	pm->setNumTVerts(count*9);
//	pm->setNumTVFaces(count*4);
	int i;


	if (!GenerateW)
		{
		int ctv = 0;
		pm->setNumTVerts(NumberStrands);
  		for (i=0;i<NumberStrands;i++)
//			if (Starting[i].on)
				{
				pm->setTVert(ctv,Starting[i].uvw);
				ctv++;
				}

		}
	else
		{
		pm->setNumTVerts(count);
		for (i=0; i<NumberStrands; i++) {
			int ct;
			ct = 0;
					

			if (Starting[NumberStrands-i-1].on)
				{
				for (int j=0;j<life;j++)
					{
					Point3 uvw;
					uvw = Starting[i].uvw;
					uvw.z = (float)(j)/(float)(life-1);
					uvw.z = 0.0f;
					pm->setTVert(ct++,Starting[i].uvw);
					}
				}
			}
		}

	
	pm->setNumTVFaces(((NumberOn*life)-NumberOn)*6+ (6*(ASegs-SegC)));


//	pm->setNumTVFaces((count-NumberStrands)*6);



	
	i = LastParticle;

	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Hair Mesh Verts %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}

//build normal strands
	for (int l=0; l<NumberStrands; l++) {
		int ct;
		ct = LastParticle-l;
		if (Starting[NumberStrands-l-1].on)
		  for (int j=0;j<life;j++)
			{
//build verts here
		// Compute this particle's size

//fix this for size and vector 

			sz = CompParticleSize(t,parts.ages[ct],life,parts.size);

		// v0 and v1 will former a basis for the plane the tetrahedron is in
			float len;
			v  = Normalize(parts.vels[ct]) * sz;
			len = sz;
			if (v!=Point3(0,1,0)) {
				v0 = Normalize(v^Point3(0,1,0))*len;
			} else {
				v0 = Point3(0.0f,0.0f,len);
				}
			v1 = Normalize(v^v0)*len;

//DebugPrint("render part %d pos  %f, %f, %f \n",ct,parts[ct].x,parts[ct].y,parts[ct].z);			

			pm->verts[ix] = (parts[ct]+v0) * tm;
			pm->verts[ix+1] = (parts[ct]+v1-v0) * tm;
			pm->verts[ix+2] = (parts[ct]-v1-v0) * tm;
			ix += 3;

			ct -= NumberStrands;
			if (ct<0)
				ct =  count+ct;
			}
		}
	
//build face list
	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Hair Mesh Faces %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}
	
	i = LastParticle;
	int face = 0;
	ix = 0;
	int ct=0;
	int MatIDct;
	int rct=0;
	for (l=0; l<NumberStrands; l++) {
//		ix = l*(life)*3;
		ix = rct*(life)*3;

		if (!GenerateW)
			ct = NumberStrands-l-1;
		MatIDct = NumberStrands-l-1;


		if (Starting[NumberStrands-l-1].on)
		  {
 		  rct++;

		  for (int j=0;j<(life-1);j++)
			{
//build faces here
			pm->faces[face].setSmGroup(1);
			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
				else pm->tvFace[face].setTVerts(ct+1,ct+1,ct);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+4,ix+3,ix);
			
//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);
			pm->faces[face].setSmGroup(1);
			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
				else pm->tvFace[face].setTVerts(ct,ct+1,ct);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+1,ix+4,ix);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);

			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
				else pm->tvFace[face].setTVerts(ct+1,ct+1,ct);
			pm->faces[face].setSmGroup(1);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+5,ix+4,ix+1);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);



			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
				else pm->tvFace[face].setTVerts(ct,ct+1,ct);
			pm->faces[face].setSmGroup(1);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+2,ix+5,ix+1);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);
		
			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
				else pm->tvFace[face].setTVerts(ct+1,ct+1,ct);
			pm->faces[face].setSmGroup(1);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+3,ix+5,ix+2);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);

			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
				else pm->tvFace[face].setTVerts(ct,ct+1,ct);
			pm->faces[face].setSmGroup(1);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix,ix+3,ix+2);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);

			if (GenerateW) ct++;
			ix += 3;

			}
		  }
		}

	

//build filler strands
	int Mult;
	int LastFace,LastVertex;
	pblock->GetValue(PB_FILLER_MULTIPLIER,t,Mult,FOREVER);	
	LastFace = ((NumberOn*life)-NumberOn)*6;
	LastVertex = (NumberOn*life*3);
	ix = 0;
	ct = 0;
	for (l=0; l<NumberStrands; l++) {
		int ct;
		if (Starting[NumberStrands-l-1].use_filler)
			{
//loop through mult
		    for (int k=0;k<Mult;k++)
				{
//get length
//	Tab<Point3> OffsetList;
//	Tab<int> SegmentList;
//				ix = ct*3*life;


				ct = LastParticle-l;

				for (int j=0;j<FillerData[NumberStrands-l-1].SegmentList[k];j++)
					{
//build verts here
		// Compute this particle's size

					sz = CompParticleSize(t,parts.ages[ct],FillerData[NumberStrands-l-1].SegmentList[k],parts.size);
//					sz = CompParticleSize(t,parts.ages[ct],life,parts.size);

		// v0 and v1 will former a basis for the plane the tetrahedron is in
					float len;
					v  = Normalize(parts.vels[ct]) * sz;
					len = sz;
					if (v!=Point3(0,1,0)) {
						v0 = Normalize(v^Point3(0,1,0))*len;
					} else {
						v0 = Point3(0.0f,0.0f,len);
						}
					v1 = Normalize(v^v0)*len;

//DebugPrint("render part %d pos  %f, %f, %f \n",ct,parts[ct].x,parts[ct].y,parts[ct].z);			

					pm->verts[LastVertex++] = (parts[ct]+v0) * tm+FillerData[NumberStrands-l-1].OffsetList[k];
					pm->verts[LastVertex++] = (parts[ct]+v1-v0) * tm+FillerData[NumberStrands-l-1].OffsetList[k];
					pm->verts[LastVertex++] = (parts[ct]-v1-v0) * tm+FillerData[NumberStrands-l-1].OffsetList[k];
					ix += 3;

					ct -= NumberStrands;
					if (ct<0)
						ct =  count+ct;

//DebugPrint("render part %d pos  %f, %f, %f \n",ct,parts[ct].x,parts[ct].y,parts[ct].z);			
//					pm->verts[LastVertex++] = pm->verts[ix]+FillerData[NumberStrands-l-1].OffsetList[k];
//					pm->verts[LastVertex++] = pm->verts[ix+1]+FillerData[NumberStrands-l-1].OffsetList[k];
//					pm->verts[LastVertex++] = pm->verts[ix+2]+FillerData[NumberStrands-l-1].OffsetList[k];
//					ix += 3;
					}

				}
		     }
//		if (Starting[NumberStrands-l-1].on)
//		  	  ct++;


		}
	
	assert(LastVertex==totalvertex);
//build filler face list
	
	i = LastParticle;
	face = 0;
//	ix = LastVertex;
	ix = (NumberOn*life*3);
	ct=0;
	MatIDct=0;
	rct=0;
	int TempLastFace = LastFace;
	for (l=0; l<NumberStrands; l++) {
//		ix = l*(life)*3;
//		ix = rct*(life)*3;
		if (!GenerateW)
			ct = NumberStrands-l-1;
		MatIDct = NumberStrands-l-1;


		if (Starting[NumberStrands-l-1].use_filler)
		  {
//		  ix = LastVertex + rct*(life)*3;
 //		  rct++;
		  for (int k=0;k<Mult;k++)
			  {

//			  face = l*6*life
			  for (int j=0;j<(FillerData[NumberStrands-l-1].SegmentList[k]-1);j++)
				{
//build faces here
				pm->faces[LastFace].setSmGroup(1);
				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
					else pm->tvFace[LastFace].setTVerts(ct+1,ct+1,ct);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+4,ix+3,ix);
			
//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);
				pm->faces[LastFace].setSmGroup(1);
				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
					else pm->tvFace[LastFace].setTVerts(ct,ct+1,ct);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+1,ix+4,ix);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);

				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
					else pm->tvFace[LastFace].setTVerts(ct+1,ct+1,ct);
				pm->faces[LastFace].setSmGroup(1);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+5,ix+4,ix+1);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);



				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
					else pm->tvFace[LastFace].setTVerts(ct,ct+1,ct);
				pm->faces[LastFace].setSmGroup(1);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+2,ix+5,ix+1);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);
		
				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
					else pm->tvFace[LastFace].setTVerts(ct+1,ct+1,ct);
				pm->faces[LastFace].setSmGroup(1);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+3,ix+5,ix+2);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);

				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
					else pm->tvFace[LastFace].setTVerts(ct,ct+1,ct);
				pm->faces[LastFace].setSmGroup(1);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix,ix+3,ix+2);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);

				if (GenerateW) ct++;
				ix += 3;

				}
			ix += 3;
			}
		}
	}


	assert(ix==totalvertex);
	assert(LastFace==totalface);

	if (Tempip != NULL)
		{

	     Tempip->PopPrompt();
		 Tempip->PopPrompt();
		}

	if (needDel) delete (EmitterMesh);

	mesh.InvalidateGeomCache();
	needDelete = TRUE;

//	FillerData.Append(1,&TempFiller,1);
	for (int fcount=0;fcount<FillerData.Count();fcount++)
		{
		FillerData[fcount].OffsetList.ZeroCount();
		FillerData[fcount].SegmentList.ZeroCount();
		}

	FillerData.ZeroCount();

	return pm;
	}


Mesh *SnowParticle::GetHairOneSidedMesh(
		TimeValue t, INode *inode, View& view, BOOL& needDelete)
	
	{
	Mesh *pm = new Mesh;
//	Point3 v, v0, v1;
	int ix=0, nx=0, count, type;
	Matrix3 tm = Inverse(inode->GetObjTMAfterWSM(t));
	TimeValue life;
	float sz;
	int GenerateW;
	char Prompt[80];

	Matrix3 cam = Inverse(view.worldToView);
	Point3 v, v0,v1, camV = cam.GetRow(3);	

	BOOL needDel;
	NullView nullView;


	pblock->GetValue(PB_RENDER,t,type,FOREVER);	
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);

	if (TestAFlag(A_RENDER)) 
		life = life *2;


	pblock->GetValue(PB_GENERATEW,t,GenerateW,FOREVER);



	if (life<=0) life = 1;

	Update(t,inode);

 	count = CountLive();

	if (count != (NumberStrands*life))
		{
		pm->setNumFaces(0);
		pm->setNumVerts(0);
		mesh.InvalidateGeomCache();
		needDelete = TRUE;
		return pm;
		}

	ObjectState os = ObjectNode->EvalWorldState(t);
	assert(os.obj->SuperClassID()==GEOMOBJECT_CLASS_ID);

	Mesh *EmitterMesh = ((GeomObject*)os.obj)->GetRenderMesh(t,ObjectNode,nullView,needDel);

	Face *FaceList = EmitterMesh->faces;






	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building One Sided Mesh %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}


	Matrix3 tm2 = ObjectNode->GetObjTMAfterWSM(tvalid);
	int ASegs = 0;
	int SegC = 0;
	int NumberOn = RemoveStrands(view,t,tm2,ASegs,SegC);
//	int NumberOn = RemoveStrands(view,t,tm2);

	int totalface,totalvertex;

	totalface = ((NumberOn*life)-NumberOn)*2 + (2*(ASegs-SegC));
	totalvertex = (NumberOn*life*2)+ (2*(ASegs));

	pm->setNumFaces(((NumberOn*life)-NumberOn)*2 + (2*(ASegs-SegC)));
	pm->setNumVerts((NumberOn*life*2)+ (2*(ASegs)));

//	pm->setNumFaces(((NumberOn*life)-NumberOn)*2);
//	pm->setNumVerts((NumberOn*life)*2);



	if (Tempip != NULL)
		{

		Tempip->PopPrompt();
		}

//	DebugPrint("Number Strands %d, face %d verts %d\n",NumberStrands,(count-NumberStrands)*6,count*3);

//	pm->setNumTVerts(count*9);
//	pm->setNumTVFaces(count*4);
	int i;
	if (!GenerateW)
		{
		pm->setNumTVerts(NumberStrands);
		for (i=0;i<NumberStrands;i++)
			pm->setTVert(i,Starting[i].uvw);
		}
	else
		{
		pm->setNumTVerts(count);
		for (i=0; i<NumberStrands; i++) {
			int ct;
			ct = 0;
			for (int j=0;j<life;j++)
				{
				Point3 uvw;
				uvw = Starting[i].uvw;
				uvw.z = (float)(j)/(float)(count/NumberStrands-1);
				uvw.z = 0.0f;
				pm->setTVert(ct++,Starting[i].uvw);
				}
			}
		}


	pm->setNumTVFaces(((NumberOn*life)-NumberOn)*2+ (2*(ASegs-SegC)));
	
//	pm->setNumTVFaces(((NumberOn*life)-NumberOn)*2);

	i = LastParticle;

	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Hair Mesh Verts %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}


	for (int l=0; l<NumberStrands; l++) {
		int ct;
		ct = LastParticle-l;
		if (Starting[NumberStrands-l-1].on)
		 {
		  for (int j=0;j<life;j++)
			{
//build verts here
		// Compute this particle's size
			sz = CompParticleSize(t,parts.ages[ct],life,parts.size);

//DebugPrint("render part %d pos  %f, %f, %f \n",ct,parts[ct].x,parts[ct].y,parts[ct].z);			
		// Compute a vector from the particle to the camera
			v  = Normalize(camV-parts[ct]);		
			v0 = Normalize(Point3(0,0,1)^v) * sz;
			v1 = Normalize(v0^v) * sz;
		
			pm->verts[ix  ] = (parts[ct]+v0+v1) * tm;
			pm->verts[ix+1] = (parts[ct]-v0+v1) * tm;
//			pm->verts[ix+2] = (parts[i]-v0-v1) * tm;
//			pm->verts[ix+3] = (parts[i]+v0-v1) * tm;



/*			pm->verts[ix] = (parts[ct]+v0) * tm;
			pm->verts[ix+1] = (parts[ct]+v1-v0) * tm;
			pm->verts[ix+2] = (parts[ct]-v1-v0) * tm;
*/			ix += 2;

			ct -= NumberStrands;
			if (ct<0)
				ct =  count+ct;
			}
		  }
		}
	
//build face list
	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Hair Mesh Faces %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}
	
	i = LastParticle;
	int face = 0;
	ix = 0;
	int ct = 0;
	int MatIDct = 0;
	int rct = 0;
	for (l=0; l<NumberStrands; l++) {
//		ix = l*life*2;
		ix = rct*life*2;
		if (!GenerateW)
			ct = NumberStrands-l-1;

		MatIDct = NumberStrands-l-1;

		if (Starting[NumberStrands-l-1].on)
		 {
 		  rct++;
		  for (int j=0;j<(life-1);j++)
			{
//build faces here
			pm->faces[face].setSmGroup(1);
			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
			else pm->tvFace[face].setTVerts(ct,ct+1,ct);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+1,ix+2,ix);
			

//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);
			pm->faces[face].setSmGroup(1);
			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
			else pm->tvFace[face].setTVerts(ct+1,ct+1,ct);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face++].setVerts(ix+3,ix+2,ix+1);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);
			if (GenerateW)
				ct++;

			ix += 2;

			}
		  }
		}


//build filler strands here
//build filler strands
	int Mult;
	int LastFace,LastVertex;
	pblock->GetValue(PB_FILLER_MULTIPLIER,t,Mult,FOREVER);	
	LastFace = ((NumberOn*life)-NumberOn)*2;
	LastVertex = (NumberOn*life*2);
	ix = 0;
	ct = 0;
	for (l=0; l<NumberStrands; l++) {
		int ct;
		if (Starting[NumberStrands-l-1].use_filler)
			{
//loop through mult
		    for (int k=0;k<Mult;k++)
				{
//get length
//	Tab<Point3> OffsetList;
//	Tab<int> SegmentList;
//				ix = ct*2*life;

				ct = LastParticle-l;


				for (int j=0;j<FillerData[NumberStrands-l-1].SegmentList[k];j++)
					{
//build verts here
		// Compute this particle's size

					sz = CompParticleSize(t,parts.ages[ct],FillerData[NumberStrands-l-1].SegmentList[k],parts.size);

//DebugPrint("render part %d pos  %f, %f, %f \n",ct,parts[ct].x,parts[ct].y,parts[ct].z);			
		// Compute a vector from the particle to the camera
					v  = Normalize(camV-parts[ct]);		
					v0 = Normalize(Point3(0,0,1)^v) * sz;
					v1 = Normalize(v0^v) * sz;
		
					pm->verts[LastVertex++] = (parts[ct]+v0+v1) * tm+FillerData[NumberStrands-l-1].OffsetList[k];
					pm->verts[LastVertex++] = (parts[ct]-v0+v1) * tm+FillerData[NumberStrands-l-1].OffsetList[k];
					ct -= NumberStrands;
					if (ct<0)
						ct =  count+ct;


//DebugPrint("render part %d pos  %f, %f, %f \n",ct,parts[ct].x,parts[ct].y,parts[ct].z);			
/*					pm->verts[LastVertex++] = pm->verts[ix]+FillerData[NumberStrands-l-1].OffsetList[k];
					pm->verts[LastVertex++] = pm->verts[ix+1]+FillerData[NumberStrands-l-1].OffsetList[k];
					ix += 2;
*/
					}

				}
		     }
//		if (Starting[NumberStrands-l-1].on)
//		  	  ct++;


		}
	
	assert(LastVertex==totalvertex);
//build filler face list
	
	i = LastParticle;
	face = 0;
//	ix = LastVertex;
	ix = (NumberOn*life*2);
	ct=0;
	MatIDct=0;
	rct=0;
	int TempLastFace = LastFace;
	for (l=0; l<NumberStrands; l++) {
//		ix = l*(life)*3;
//		ix = rct*(life)*3;
		if (!GenerateW)
			ct = NumberStrands-l-1;
		MatIDct = NumberStrands-l-1;


		if (Starting[NumberStrands-l-1].use_filler)
		  {
//		  ix = LastVertex + rct*(life)*3;
 //		  rct++;
		  for (int k=0;k<Mult;k++)
			  {

//			  face = l*6*life
			  for (int j=0;j<(FillerData[NumberStrands-l-1].SegmentList[k]-1);j++)
				{
//build faces here

//build faces here
				pm->faces[LastFace].setSmGroup(1);
				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
				else pm->tvFace[LastFace].setTVerts(ct,ct+1,ct);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+1,ix+2,ix);
			

//DebugPrint("Face %d a %d b %d c %d\n",face,ix,ix+1,ix+NumberStrands*3);
				pm->faces[LastFace].setSmGroup(1);
				if (!GenerateW)
					pm->tvFace[LastFace].setTVerts(ct,ct,ct);
				else pm->tvFace[LastFace].setTVerts(ct+1,ct+1,ct);
				pm->faces[LastFace].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
				pm->faces[LastFace++].setVerts(ix+3,ix+2,ix+1);
//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);



				if (GenerateW) ct++;
				ix += 2;

				}
			ix += 2;
			}
		}
	}


	assert(ix==totalvertex);
	assert(LastFace==totalface);




	if (Tempip != NULL)
		{
	
	     Tempip->PopPrompt();
		 Tempip->PopPrompt();
		}

	if (needDel) delete (EmitterMesh);
	mesh.InvalidateGeomCache();
	needDelete = TRUE;

	for (int fcount=0;fcount<FillerData.Count();fcount++)
		{
		FillerData[fcount].OffsetList.ZeroCount();
		FillerData[fcount].SegmentList.ZeroCount();
		}

	FillerData.ZeroCount();

	return pm;
	}


Mesh *SnowParticle::GetWireMesh(
		TimeValue t, INode *inode, View& view, BOOL& needDelete)
	
	{
	Mesh *pm = new Mesh;
//	Point3 v, v0, v1;
	int ix=0, nx=0, count, type;
	Matrix3 tm = Inverse(inode->GetObjTMAfterWSM(t));
	TimeValue life;
//	float sz;
	int GenerateW;
	char Prompt[80];

	Matrix3 cam = Inverse(view.worldToView);
	Point3 v, v0,v1, camV = cam.GetRow(3);	

	BOOL needDel;
	NullView nullView;

	ObjectState os = ObjectNode->EvalWorldState(t);
	assert(os.obj->SuperClassID()==GEOMOBJECT_CLASS_ID);

	Mesh *EmitterMesh = ((GeomObject*)os.obj)->GetRenderMesh(t,ObjectNode,nullView,needDel);

	Face *FaceList = EmitterMesh->faces;



	pblock->GetValue(PB_RENDER,t,type,FOREVER);	
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);

	if (TestAFlag(A_RENDER)) 
		life = life *2;


	pblock->GetValue(PB_GENERATEW,t,GenerateW,FOREVER);



	if (life<=0) life = 1;

	Update(t,inode);

 	count = CountLive();

	if (count != (NumberStrands*life))
		{
		pm->setNumFaces(0);
		pm->setNumVerts(0);
		mesh.InvalidateGeomCache();
		needDelete = TRUE;
		return pm;
		}



	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Wire Mesh %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}

	Matrix3 tm2 = ObjectNode->GetObjTMAfterWSM(tvalid);
	int ASegs = 0;
	int CSegs = 0;
	int NumberOn = RemoveStrands(view,t,tm2,ASegs,CSegs);
//	int NumberOn = RemoveStrands(view,t,tm2);


	pm->setNumFaces(((NumberOn*life)-NumberOn));
	pm->setNumVerts((NumberOn*life)+1);

//	pm->setNumFaces((count-NumberStrands));
//	pm->setNumVerts(count+1);



	if (Tempip != NULL)
		{

		Tempip->PopPrompt();
		}

	int i;
	if (!GenerateW)
		{
		pm->setNumTVerts(NumberStrands);
		for (i=0;i<NumberStrands;i++)
			pm->setTVert(i,Starting[i].uvw);
		}
	else
		{
		pm->setNumTVerts(count);
		for (i=0; i<NumberStrands; i++) {
			int ct;
			ct = 0;
			for (int j=0;j<life;j++)
				{
				Point3 uvw;
				uvw = Starting[i].uvw;
				uvw.z = (float)(j)/(float)(count/NumberStrands-1);
				uvw.z = 0.0f;
				pm->setTVert(ct++,Starting[i].uvw);
				}
			}
		}

	pm->setNumTVFaces((NumberOn*life)-NumberOn);
//	pm->setNumTVFaces((count-NumberStrands));

	i = LastParticle;

	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Hair Mesh Verts %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}


	pm->verts[0] = (parts[0]) * tm;
	ix = 1;
	for (int l=0; l<NumberStrands; l++) {
		int ct;
		ct = LastParticle-l;
		if (Starting[NumberStrands-l-1].on)
		  for (int j=0;j<life;j++)
			{
//build verts here
		// Compute this particle's size
			pm->verts[ix  ] = (parts[ct]) * tm;
			ix += 1;

			ct -= NumberStrands;
			if (ct<0)
				ct =  count+ct;
			}
		}
	
//build face list
	if (Tempip != NULL)
		{

		sprintf(Prompt,"Building Hair Mesh Faces %d",tvalid);
		Tempip->PushPrompt(_T(Prompt));
		}
	
	i = LastParticle;
	int face = 0;
	ix = 1;
	int ct = 0;
	int MatIDct = 0;
	int rct = 0;
	for (l=0; l<NumberStrands; l++) {
//		ix = l*(count/NumberStrands)+1;
		ix = rct*life+1;
		if (!GenerateW)
			ct = NumberStrands-l-1;

		MatIDct = NumberStrands-l-1;
		if ((Starting[NumberStrands-l-1].on))
		  {
		  rct++;
		  for (int j=0;j<(life-1);j++)
			{
//build faces here
			pm->faces[face].setSmGroup(1);
			if (!GenerateW)
				pm->tvFace[face].setTVerts(ct,ct,ct);
			else pm->tvFace[face].setTVerts(ct,ct+1,ct);
			pm->faces[face].setMatID(FaceList[Starting[MatIDct].AttachedToFace].getMatID());
			pm->faces[face].setVerts(ix,ix+1,0);
			pm->faces[face].setEdgeVis(0,EDGE_VIS);
			pm->faces[face].setEdgeVis(1,EDGE_INVIS);
			pm->faces[face++].setEdgeVis(2,EDGE_INVIS);
			

//DebugPrint("Face %d a %d b %d c %d\n",face,ix+1,ix+NumberStrands*3+1,ix+NumberStrands*3);
			if (GenerateW)
				ct++;

			ix += 1;

			}
		  }
		}
	if (Tempip != NULL)
		{
	
	     Tempip->PopPrompt();
		 Tempip->PopPrompt();
		}

	if (needDel) delete (EmitterMesh);
	mesh.InvalidateGeomCache();
	needDelete = TRUE;
	return pm;
	}


Mesh *SnowParticle::GetNullMesh(
		TimeValue t, INode *inode, View& view, BOOL& needDelete)
	
	{
	Mesh *pm = new Mesh;
	pm->setNumFaces(0);
	pm->setNumVerts(0);
	mesh.InvalidateGeomCache();
	needDelete = TRUE;
	return pm;

	}


Mesh* SnowParticle::GetRenderMesh(TimeValue t, INode *inode, View& view, BOOL& needDelete)
	{
	Mesh *pm = new Mesh;		
	BOOL single = FALSE;	
	float size, tumble, scale, sz;
	int type, count;
	TimeValue life;	

	int Go;
	if (TestAFlag(A_RENDER))
		{
	     pblock->GetValue(PB_RENDERACTIVE,t,Go,FOREVER);
		 if (!Go) return GetNullMesh(t,inode,view,needDelete);
		}
	else
		{
	     pblock->GetValue(PB_VIEWACTIVE,t,Go,FOREVER);
		 if (!Go) 
			 return GetNullMesh(t,inode,view,needDelete);
			
		}

	if (ObjectNode==NULL) return GetNullMesh(t,inode,view,needDelete);


	pblock->GetValue(PB_DROPSIZE,t,size,FOREVER);
	pblock->GetValue(PB_TUMBLE,t,tumble,FOREVER);
	pblock->GetValue(PB_SCALE,t,scale,FOREVER);
	pblock->GetValue(PB_RENDER,t,type,FOREVER);
	pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);

	if (TestAFlag(A_RENDER)) 
		life = life *2;


	if (life<=0) life = 1;
	scale /= 50.0f;
	single = type==RENDTYPE2;
	if (type==RENDTYPE3) return GetFacingRenderMesh(t,inode,view,needDelete);
	else if (type==RENDTYPE5) return GetPlusMesh(t,inode,view,needDelete);
	else if (type==RENDTYPE6) return GetHairMesh(t,inode,view,needDelete);
	else if (type==RENDTYPE7) return GetHairOneSidedMesh(t,inode,view,needDelete);
	else if (type==RENDTYPE8) return GetWireMesh(t,inode,view,needDelete);

	Matrix3 tm = Inverse(inode->GetObjTMAfterWSM(t));
	int ix=0, nx=0, numV = single ? 3 : 6, numF = single ? 1 : 2;

	Update(t,inode);
	count = CountLive();

	if (!single) {
		pm->setNumFaces(count*2);
		pm->setNumVerts(count*6);
		pm->setNumTVerts(count*6);
		pm->setNumTVFaces(count*2);
	} else {
		pm->setNumFaces(count);
		pm->setNumVerts(count*3);
		pm->setNumTVerts(count*3);
		pm->setNumTVFaces(count);
		}

	for (int i=0; i<parts.Count(); i++) {
		if (!parts.Alive(i)) continue;
		
		// Compute this particle's size
		sz = CompParticleSize(t,parts.ages[i],life,size);

		for (int j=0; j<numV; j++) {
			pm->tVerts[ix+j].x = pm->verts[ix+j].x = 
				(float)cos(TWOPI*float(j)/float(numV));
			pm->tVerts[ix+j].z = pm->verts[ix+j].y = 0.0f;
			pm->tVerts[ix+j].y = pm->verts[ix+j].z = 
				(float)sin(TWOPI*float(j)/float(numV));
			pm->verts[ix+j]  = (pm->verts[ix+j]*TumbleMat(i,tumble,scale)) * sz;
			pm->verts[ix+j]  += parts[i];
			pm->tVerts[ix+j] *= 0.5f;
			pm->tVerts[ix+j] += Point3(0.5f,0.5f,0.0f);
			
			// Particles are in world space so we transform them back
			// into object space because that's what the renderer expects.
			pm->verts[ix+j] = tm * pm->verts[ix+j];
			}
		
		if (single) {
			pm->faces[nx].setSmGroup(0);
			pm->faces[nx].setVerts(ix,ix+1,ix+2);
			pm->faces[nx].setMatID((MtlID)i);
			pm->tvFace[nx].setTVerts(ix,ix+1,ix+2);
		} else {
			pm->faces[nx  ].setSmGroup(0);
			pm->faces[nx  ].setVerts(ix  ,ix+2,ix+4);
			pm->faces[nx  ].setMatID((MtlID)i);
			pm->faces[nx+1].setSmGroup(0);
			pm->faces[nx+1].setVerts(ix+1,ix+3,ix+5);
			pm->faces[nx+1].setMatID((MtlID)i);
			pm->tvFace[nx  ].setTVerts(ix  ,ix+2,ix+4);
			pm->tvFace[nx+1].setTVerts(ix+1,ix+3,ix+5);
			}
		
		ix += numV;
		nx += numF;
		}
	
	mesh.InvalidateGeomCache();
	needDelete = TRUE;
	return pm;
	}





static float CompViewParticleSize(
		TimeValue age, TimeValue life, float size)
	{

	float u;
	u = float(age)/float(life);
	return (1.0f-u) * size;

//	if (u>0.75f) {
//		return (1.0f-smoothstep(0.75f,1.0f,u)) * size;
//	} else {
//		return size;
//		}
	}



BOOL SnowParticleDraw::DrawParticle(
		GraphicsWindow *gw,ParticleSys &parts,int i)
	{
	Point3 pt[3];	
	float size = CompViewParticleSize(parts.ages[i],life,parts.size);
	Matrix3 mat = obj->TumbleMat(i,tumble,scale);
	Point3 vx, vy, vz;

	pt[0] = parts[i];
	int it;
	it = i + NParticles;
	if (it >= Alive) it = (it)%NParticles;
//need to check for first particle
//DebugPrint("Point %d to %d last %d count %d \n",i,it,LParticle,NParticles);
	pt[1] = parts[it];




	if (!((i<=(LParticle)) && (i>(LParticle-NParticles))))
		{
		gw->setColor(LINE_COLOR, 1.0f,1.0f,1.0f);
		gw->polyline(2,pt,NULL,NULL,FALSE,NULL);
//DebugPrint("DrawingParticle");
		}
//DebugPrint("\n");

	if ((i<NParticles) && (ptype ==3))
		{
		gw->setColor(LINE_COLOR, 0.0f,0.0f,1.0f);
		pt[0] = parts[i];
		gw->marker(pt,HOLLOW_BOX_MRKR);
		}
		else 
		{
		gw->setColor(LINE_COLOR, 1.0f,0.0f,0.0f);
		pt[0] = parts[i];
		gw->marker(pt,POINT_MRKR);
		}
	return FALSE;

	}

MarkerType SnowParticle::GetMarkerType() 
	{
	int type;
	pblock->GetValue(PB_DISPTYPE,0,type,FOREVER);	



	switch (type) {
		case 0: {			
			theSnowDraw.ptype = 0;
			theSnowDraw.obj = this;
			pblock->GetValue(PB_SEGCOUNT,0,theSnowDraw.life,FOREVER);
			if (TestAFlag(A_RENDER)) 
				theSnowDraw.life = theSnowDraw.life *2;

			if (theSnowDraw.life<=0) theSnowDraw.life = 1;
			parts.SetCustomDraw(&theSnowDraw);			
			return POINT_MRKR;
			}
		case 3: {			
			theSnowDraw.ptype = 3;
			theSnowDraw.obj = this;
			pblock->GetValue(PB_SEGCOUNT,0,theSnowDraw.life,FOREVER);
			if (TestAFlag(A_RENDER)) 
				theSnowDraw.life = theSnowDraw.life *2;

			if (theSnowDraw.life<=0) theSnowDraw.life = 1;
			parts.SetCustomDraw(&theSnowDraw);			
			return POINT_MRKR;
			}
		case 1:
			parts.SetCustomDraw(NULL);
			return POINT_MRKR;			
		case 2:
			parts.SetCustomDraw(NULL);
			return PLUS_SIGN_MRKR;
		default:
			return PLUS_SIGN_MRKR;
		}
	}

Matrix3 SnowParticle::TumbleMat(int index,float amount, float scale)
	{
	Matrix3 mat;
	Quat q;
	float ang[3];

	srand(int(PARTICLE_SEED) * Perm(index) + int(PARTICLE_SEED));
	
	for (int i=0; i<3; i++) {
		ang[i] = (float(2*rand())/float(RAND_MAX) - 1.0f);
		if (amount>0.0f) {
			float off = 8725.0f*i;
			ang[i] += noise3((parts[index]+Point3(off,off,off))*scale)*amount;
			}
		ang[i] *= TWOPI;
		}
	
	EulerToQuat(ang,q);
	q.MakeMatrix(mat);
	return mat;
	}






