
#include "prim.h"
#include "iparamm.h"
#include "Simpobj.h"
#include "texutil.h"
#include "istdplug.h"
#include "decomp.h"
#include "trig.h"
#include "bmmlib.h"
#include "splshape.h"
#include "simpspl.h"

#include "hair.h"


Point3 GenParticle::GetNormal(RVertex *n)

{
  Point3 vn(0.0f,0.0f,0.0f);
  DWORD NCount;
  DWORD i;

  NCount = (n->rFlags & NORCT_MASK);

  if (NCount ==1)
	{
	vn = n->rn.getNormal();
	}
	else
	{
	for (i=0;i<NCount;i++)
		{
		 vn += n->ern[i].getNormal();
		}
	}


  return vn/(float)NCount;
}



int GenParticle::AddPoints(Point3 v1, Point3 n1, Point3 TV1, Point3 v2, Point3 n2, Point3 TV2, Point3 v3, Point3 n3, Point3 TV3, int depth, int face)

{
 Point3 centroid,EdgeA,EdgeB,EdgeC;

 centroid = (v1+v2+v3)/3.0f;
// EdgeA = (v1+v2)/2.0f;
// EdgeB = (v2+v3)/2.0f;
// EdgeC = (v3+v1)/2.0f;

AddPoint(centroid,(n1+n2+n3)/3.0f,(TV1+TV2+TV3)/3.0f,FALSE,face);


 depth--;

 if (depth != 0)
	{
	AddPoints(v1,        n1,      TV1,
		      v2,        n2,      TV2,
			  centroid,	(n1+n2+n3)/3.0f,(TV1+TV2+TV3)/3.0f,
			  depth,face);
	
	AddPoints(v2,        n2,      TV2,
		      v3,        n3,      TV3,
			  centroid,	(n1+n2+n3)/3.0f,(TV1+TV2+TV3)/3.0f,
			  depth,face);

	AddPoints(v3,        n3,      TV3,
		      v1,        n1,      TV1,
			  centroid,	(n1+n2+n3)/3.0f,(TV1+TV2+TV3)/3.0f,
			  depth,face);
	}
return 1;

}


int GenParticle::AddPoint(Point3 v1, Point3 n, Point3 uvw, BOOL check, int face)

{
 int i,found;
 VertexType temp;


 found = 0;

 if (check)
	{
	 for (i =0; i< DVerts.Count();i++)
		{
		 if (DVerts[i].Pos==v1)
			{
			 found =1 ;
			 i = DVerts.Count();
			}
		}
	}
 if (!found)
	{
	temp.Pos = v1;
	temp.pNormal = n;
	temp.uvw = uvw;
	temp.used = 0;
	temp.AttachedToFace = face;
	DVerts.Append(1,&temp,1);
	}
 return 1;

}


BOOL GenParticle::IsVertSelected(int VertID, Mesh *mesh, int& face)

{

Face* list = mesh->faces;
int FaceCount = mesh->getNumFaces();
BitArray SelectedFaces = mesh->FaceSel();
int a,b,c;
for (int i=0;i<FaceCount;i++)
	{
	face = i;
	if (SelectedFaces[i])
		{
		a = list[i].v[0];
		b = list[i].v[1];
		c = list[i].v[2];
		if (a==VertID) return 1;
		if (b==VertID) return 2;
		if (c==VertID) return 3;
		}
	}
return FALSE;
}

void GenParticle::BuildVerts(TimeValue t)

{

	BOOL needDel;
	NullView nullView;
	int i;
	char Prompt[80];

	ObjectState os = ObjectNode->EvalWorldState(t);
	assert(os.obj->SuperClassID()==GEOMOBJECT_CLASS_ID);

	Mesh *mesh = ((GeomObject*)os.obj)->GetRenderMesh(t,ObjectNode,nullView,needDel);

//	if (!mesh) return;
//compute vertices
	Point3 *VertList =  mesh->verts;
	Face *FaceList = mesh->faces;

	TVFace	*TVFaceList = mesh->tvFace;



//	if (!mesh->normalsBuilt)
//		{
		mesh->buildNormals();
//		}


	int UseSelectedFaces;


	pblock->GetValue(PB_USE_SELECTED_FACES,t, UseSelectedFaces,FOREVER);
	
	int TDepth = 0;
	int D;
	int NumberVerts,NumberVertsNeeded,CFaces;
	int NumberSelectedFaces;
	BitArray SelFaces = mesh->FaceSel();
	NumberSelectedFaces = SelFaces.NumberSet();

	if (NumberSelectedFaces == 0)  UseSelectedFaces = 0;

	
	NumberVerts = mesh->numVerts;
	if (UseSelectedFaces)
		CFaces = NumberSelectedFaces;
		else CFaces = mesh->numFaces; 
	NumberVertsNeeded = NumberStrands;


//check for distribution list


	while (NumberVertsNeeded>NumberVerts)
		{
		TDepth++;
		NumberVerts += CFaces;
		CFaces *= 3;
		}


	BOOL done = FALSE;

//get selected face


	while (!done)
		{
		DVerts.ZeroCount();
		
//		DVerts.Resize(NumberVerts);


		Point3 n1(0.0f,0.0f,0.0f),n2(0.0f,0.0f,0.0f),n3(0.0f,0.0f,0.0f);
 		for (i=0;i<mesh->numVerts;i++)
			{
//if vert is part of selected face
			if (UseSelectedFaces)

				{
				int f;
				int abc = IsVertSelected(i,mesh,f);
				if (abc)
					{
					AddPoint(VertList[i],n1,n1,FALSE,0);
					n1 = GetNormal(mesh->getRVertPtr(FaceList[f].v[abc-1]));
//Tempip->PopPrompt();

					Point3 TVertA(0.0f,0.0f,0.0f);
					if  (FaceList[f].flags & HAS_TVERTS)   
						{
//sprintf(Prompt,"Getting TVerts %d",i);
//Tempip->PushPrompt(_T(Prompt));
						TVertA = (Point3) mesh->getTVert(TVFaceList[f].t[abc-1]);
//Tempip->PopPrompt();
						}

					int ct = DVerts.Count()-1;
					DVerts[ct].pNormal = n1;
 
					DVerts[ct].uvw = TVertA;

					DVerts[ct].AttachedToFace = f;
	
					}
				}
			else AddPoint(VertList[i],n1,n1,FALSE,0);
			}

		sprintf(Prompt,"Tessalating Object depth %d  time %d Added reg verts",TDepth,t);
		if (Tempip != NULL)
			{
		 	Tempip->PushPrompt(_T(Prompt));
			}

		int ActiveFace = 0;
	 	for (i=0;i<mesh->numFaces;i++)
			{
//if vert is part of selected face

//sprintf(Prompt,"Getting Normals %d",i);
//Tempip->PushPrompt(_T(Prompt));
			if (((UseSelectedFaces) && (SelFaces[i]))  || (!UseSelectedFaces))
				{

				n1 = GetNormal(mesh->getRVertPtr(FaceList[i].v[0]));
				n2 = GetNormal(mesh->getRVertPtr(FaceList[i].v[1]));
				n3 = GetNormal(mesh->getRVertPtr(FaceList[i].v[2]));
//Tempip->PopPrompt();

				Point3 TVertA(0.0f,0.0f,0.0f),TVertB(0.0f,0.0f,0.0f),TVertC(0.0f,0.0f,0.0f);
				if  (FaceList[i].flags & HAS_TVERTS)   
					{
//sprintf(Prompt,"Getting TVerts %d",i);
//Tempip->PushPrompt(_T(Prompt));
					TVertA = (Point3) mesh->getTVert(TVFaceList[i].t[0]);
					TVertB = (Point3) mesh->getTVert(TVFaceList[i].t[1]);
					TVertC = (Point3) mesh->getTVert(TVFaceList[i].t[2]);
//Tempip->PopPrompt();
					}
				if (!UseSelectedFaces)
					{


					DVerts[FaceList[i].v[0]].pNormal = n1;
					DVerts[FaceList[i].v[1]].pNormal = n2;
					DVerts[FaceList[i].v[2]].pNormal = n3;
 
					DVerts[FaceList[i].v[0]].uvw = TVertA;
					DVerts[FaceList[i].v[1]].uvw = TVertB;
					DVerts[FaceList[i].v[2]].uvw = TVertC;

					DVerts[FaceList[i].v[0]].AttachedToFace = i;
					DVerts[FaceList[i].v[1]].AttachedToFace = i;
					DVerts[FaceList[i].v[2]].AttachedToFace = i;
					}
	

				if (TDepth >0)
					{
					D=TDepth;
					AddPoints(VertList[FaceList[i].v[0]],n1,TVertA,
					      VertList[FaceList[i].v[1]],n2,TVertB,
						  VertList[FaceList[i].v[2]],n3,TVertC,

						  D,i);
					}
				}
			}

//check disribution map
		if (thebmDistribution != NULL)
			{
//check vertices
			srand(unsigned int(PARTICLE_SEED));
		 	for (i=0;i<DVerts.Count();i++)
				{
				if  (GetDistributionPixel(DVerts[i].uvw.x,DVerts[i].uvw.y)<=(float(rand())/float(RAND_MAX)))
					{
					DVerts.Delete(i,1);
					i--;
					}
				}
			}	 
		if (DVerts.Count()>=NumberVertsNeeded)
			done = TRUE;
			else 
			{
			done = FALSE;
			TDepth++;

			}

		if (Tempip != NULL)
			{

			Tempip->PopPrompt();
			}
		}

if (needDel) delete (mesh);

}


void GenParticle::BuildStartingPos(BOOL RecreateEverything,TimeValue bt)

{
int i;
StartPosClass temp;
float min,max,dif,offset;
char Prompt[80];


if (Tempip != NULL)
	{
	sprintf(Prompt,"Building Verts List %d",bt);
	Tempip->PushPrompt(_T(Prompt));
	}


BuildVerts(bt);


if (Tempip != NULL)
	{
	Tempip->PopPrompt();
	}

temp.pos.x = 0.0f;
temp.pos.y = 0.0f;
temp.pos.z = 0.0f;

temp.prevpos.x = 0.0f;
temp.prevpos.y = 0.0f;
temp.prevpos.z = 0.0f;

temp.prevnormal.x = 0.0f;
temp.prevnormal.y = 0.0f;
temp.prevnormal.z = 0.0f;


temp.vel.x = 0.0f;
temp.vel.y = 0.0f;
temp.vel.z = 0.0f;

temp.uvw.x = 0.0f;
temp.uvw.y = 0.0f;
temp.uvw.z = 0.0f;

temp.JitterRandom.x = 0.0f;
temp.JitterRandom.y = 0.0f;
temp.JitterRandom.z = 0.0f;

temp.cutted = FALSE;
temp.on = TRUE;
temp.use_filler = TRUE;

temp.seed = 0;
temp.Group = -1;
temp.StrandLength=0.0f;

pblock->GetValue(PB_LENGTH_MIN,bt,min,FOREVER);
pblock->GetValue(PB_LENGTH_MAX,bt,max,FOREVER);

dif = max-min;


//sprintf(Prompt,"Building Particle Starting Pos %d",tvalid);
//Tempip->PushPrompt(_T(Prompt));

if (Tempip != NULL)
	{
	sprintf(Prompt,"Building Starting Pos Verts %d",bt);
	Tempip->PushPrompt(_T(Prompt));
	}

if ((RecreateEverything) || (Starting.Count()==0))
	{
	Starting.ZeroCount();
	Starting.Resize(NumberStrands);
//	Starting.Append(NumberStrands,&temp,1);
	for(i=0;i<NumberStrands;i++)
		{
		Starting.Append(1,&temp,1);
		}
	}

float Jitter;
pblock->GetValue(PB_JITTER,bt, Jitter,FOREVER);

for(i=0;i<NumberStrands;i++)
	{
	 
	 
	int c;
	c = (int) ((float)DVerts.Count()*  ((float)(i%NumberStrands)/(float)NumberStrands));
	Starting[i].prevpos = Starting[i].pos;
	Starting[i].prevnormal = Starting[i].vel;
	Starting[i].pos = DVerts[c].Pos;
	Starting[i].vel = DVerts[c].pNormal; 
	Starting[i].AttachedToFace = DVerts[c].AttachedToFace; 

	if (RecreateEverything)
		{
		offset = dif*(float(rand())/float(RAND_MAX));
		Starting[i].uvw = DVerts[c].uvw; 
		
//set strand length
		Starting[i].StrandLength = 	min+offset;
//set group
		Starting[i].Group = -2;
		Starting[i].seed = (int)((float(rand())/float(RAND_MAX)) *6432078.0f);
		Starting[i].JitterRandom.x = (float(rand())/float(RAND_MAX)) * Jitter;
		Starting[i].JitterRandom.y = (float(rand())/float(RAND_MAX)) * Jitter;
		Starting[i].JitterRandom.z = (float(rand())/float(RAND_MAX)) * Jitter;
		}
	Starting[i].pos += Starting[i].JitterRandom;

	}

//build Grouping
float CChance,MaxDist;
int  cmin,cmax;
Point3 Center;
pblock->GetValue(PB_CLUMPCHANCE,bt,CChance,FOREVER);
pblock->GetValue(PB_CLUMP_MIN,bt,cmin,FOREVER);
pblock->GetValue(PB_CLUMP_MAX,bt,cmax,FOREVER);

pblock->GetValue(PB_CLUMP_RADIUS,bt,MaxDist,FOREVER);

if ((RecreateEverything) && (TestAFlag(A_RENDER)))
	{
	for(i=0;i<NumberStrands;i++)
		{

//check if mapped

//check if random
		if (((float(rand())/float(RAND_MAX)) < CChance) && (Starting[i].Group == -2))
			{
//get number of strands
			int Number;
			Tab<int> IndexList;
			Tab<float> dist;
			Number = cmin + (int)((float)(cmax-cmin) *(float(rand())/float(RAND_MAX)) );
//build list
			IndexList.ZeroCount();
			dist.ZeroCount();

			int ct = 0;
			float tdist;

			for(ct=i;ct<NumberStrands;ct++)
				{
//add if list is less than count
				if (Starting[ct].Group == -2)
					{
					if (IndexList.Count() < Number) 
						{
//add
						if (IndexList.Count() == 0)
							{
							IndexList.Append(1,&ct,1);
							Center = Starting[ct].pos;
							tdist = 0.0f;
							dist.Append(1,&tdist,1);
							}
							else
							{
							tdist = Length(Center - Starting[ct].pos);
							if (tdist<MaxDist)
								{
								IndexList.Append(1,&ct,1);
								dist.Append(1,&tdist,1);
								}
							}
						}
					else
//search through list if less than add
						{
						tdist = Length(Center - Starting[ct].pos);
						int tct;
						tct = ct;
						for (int j=0;j<IndexList.Count();j++)
							{
							if (tdist < dist[j])
								{
								int rct;
								float rdist;
								rdist = dist[j];
								dist[j] = tdist;
								tdist = rdist;
								rct = IndexList[j];
								IndexList[j] = tct;
								tct = rct;
								}
							}
						}
					}
				}
			for (ct = 0; ct < IndexList.Count();ct++)
				{
				Starting[IndexList[ct]].Group = IndexList[0];

				}
			}
		else
			{
//set group to -1 if not
			if (Starting[i].Group==-2) Starting[i].Group = -1;
			}
		}

	}



if (SplineCageNode != NULL) BuildSplineCageData(bt);


DVerts.ZeroCount();

if (SplineCageNode != NULL) BuildSplineCagePercentData(bt);


if (Tempip != NULL)
	{
	Tempip->PopPrompt();
	}

}



void GenParticle::BuildSplineCageData(TimeValue t)
{

//SplineShape *SplineCage;// = (SplineShape *)os->obj;


ObjectState os;
os = SplineCageNode->EvalWorldState(t);

ShapeObject *shape = (ShapeObject *) os.obj;
// If the shape can convert itself to a BezierShape, have it do so!

BezierShape bshape;

if(shape->CanMakeBezier())
	shape->MakeBezier(t, bshape);
else {
	PolyShape pShape;
	shape->MakePolyShape(t, pShape);
	bshape = pShape;	// UGH -- Convert it from a PolyShape -- not good!
	}


//SplineCage = (SplineShape*)os.obj;




SplineStartingVerts.ZeroCount();

//Matrix3 EmitterTM,SplineTM;
Point3 Start, vert;
float Dist;
int closest;

//EmitterTM = ObjectNode->GetObjTMAfterWSM(t);
//SplineTM = SplineCageNode->GetObjTMAfterWSM(t);

Tab<int> TempList;

SplineLengths.ZeroCount();

float sdist;


int NumberSpline = bshape.SplineCount();


for (int j=0;j<bshape.SplineCount();j++)
	{
	Spline3D *spline = bshape.GetSpline(j);

	sdist = spline->SplineLength();


	SplineLengths.Append(1,&sdist,1);

//get first spline vert
	Start = spline->GetKnotPoint(0);
//trnasfer to world space
	Start = Start * SplineTM;

//	vert = Starting[0].pos;
	vert = DVerts[0].Pos;
	vert = vert * EmitterTM;
	Dist = Length(vert-Start);
	closest = 0;

	for(int i=1;i<DVerts.Count();i++)
		{
//get node vert
		vert = DVerts[i].Pos;
//trnasfer to world space
		vert = vert * EmitterTM;

//see if close of enought to assign to vert
		float Dist2;
		Dist2 = Length(vert-Start);
		if (Dist2<Dist)
			{
			Dist = Dist2;
			closest = i;
			}
		}
	Point3 Temp;
	Temp = DVerts[closest].pNormal;
	SplineStartingVerts.Append(1,&Temp,1);
	TempList.Append(1,&closest,1);
	}


//build distance list
//DebugPrint("Spline Cage Data\n");

for(int i=0;i<Starting.Count();i++)
//for(int i=0;i<DVerts.Count();i++)
	{
//DebugPrint("Strand %d Starting Pos %4.2f %4.2f %4.2f \n",i,Starting[i].pos.x,Starting[i].pos.y,Starting[i].pos.z);
	Starting[i].DistToSpline.ZeroCount();
	for (int j=0;j<bshape.SplineCount();j++)
		{
		float d = 0.0f;
		float d2 = 0.0f;
		Point3 vert2;

//get node vert
		vert = Starting[i].pos;
//		vert = DVerts[i].Pos;
//trnasfer to world space
		vert = vert * EmitterTM;

//		vert2 = Starting[SplineStartingVerts[j]].pos;
		vert2 = DVerts[TempList[j]].Pos;

		vert2 = vert2 * EmitterTM;

		d = Length(vert2-vert); 

//DebugPrint("             Spline %d distance %4.2f \n",j,d);


		Starting[i].DistToSpline.Append(1,&d,1);
		}
	}



TempList.ZeroCount();

}

void GenParticle::BuildSplineCagePercentData(TimeValue t)
{


ObjectState os;
os = SplineCageNode->EvalWorldState(t);


ShapeObject *shape = (ShapeObject *) os.obj;
// If the shape can convert itself to a BezierShape, have it do so!

BezierShape bshape;

if(shape->CanMakeBezier())
	shape->MakeBezier(t, bshape);
else {
	PolyShape pShape;
	shape->MakePolyShape(t, pShape);
	bshape = pShape;	// UGH -- Convert it from a PolyShape -- not good!
	}



Matrix3 ETM,STM;

ETM = EmitterTM;
STM = SplineTM;






//build closest list

Tab<int> ClosestList;
int Master;
float c;
ClosestList.ZeroCount();
int UseLimits;

pblock->GetValue(PB_SPLINE_LIMITS,t,UseLimits,FOREVER);

NumberOfSplines = bshape.SplineCount();

for (int Index = 0;Index<NumberStrands;Index++)
	{




//loop through splines find closest this is the master
	Master = 0;
	c = Starting[Index].DistToSpline[0];
	int TotalSplineInfluence = 0;

//	if (Index == 38)
//		DebugPrint("Error Here\n");


	if (UseLimits)
		{	

		for (int i=1;i<bshape.SplineCount();i++)
			{
			float dist = Starting[Index].DistToSpline[i];
			if (Starting[Index].DistToSpline[i]<c)
				{
				c = Starting[Index].DistToSpline[i];
				Master = i;
				}
			}

// loop through spline
		for (i=0;i<bshape.SplineCount();i++)
			{
			Point3 vert,tvert;
			float SplineToPoint,SplineToMaster;
//    get distance from spline to point 
			SplineToPoint = Starting[Index].DistToSpline[i];


//	  get distance spline to master
			Spline3D *spline = bshape.GetSpline(Master);
			tvert = spline->GetKnotPoint(0);
			tvert = tvert * SplineTM;

			spline = bshape.GetSpline(i);
			vert = spline->GetKnotPoint(0);
			vert = vert * SplineTM;

			SplineToMaster = Length(tvert-vert);


//    if distance from spline to point is less than spline to master   
//          add to list

			if ((SplineToPoint>SplineToMaster) && (i!=Master))
				{
				Starting[Index].DistToSpline[i] = -1.0f;
				}
			else TotalSplineInfluence++;
			}
		}


	TotalSplineInfluence= 0;
	for (int i=0;i<NumberOfSplines;i++)
		{
		if (Starting[Index].DistToSpline[i]>=0.0f)
			TotalSplineInfluence++;

		}



//check Max Limit here
	int NumberInfluence;
	pblock->GetValue(PB_MAX_NUMBER_SPLINES,t,NumberInfluence,FOREVER);

	while (TotalSplineInfluence > NumberInfluence)
		{
//find farthest spline and nuke it
		int farthest = 0;
		
		for (i=1;i<NumberOfSplines;i++)
			{
			float dist = Starting[Index].DistToSpline[i];
			if (Starting[Index].DistToSpline[i]>Starting[Index].DistToSpline[farthest])
				{
				farthest = i;
				}
			}
		Starting[Index].DistToSpline[farthest]= -1.0f;
		TotalSplineInfluence--;
		}


	float d;
	float p;
	d = 0.0f;
	p = 0.0f;
	Starting[Index].PercentControl.ZeroCount();
	for (i=0;i<bshape.SplineCount();i++)
		{
		float dv;
		if (Starting[Index].DistToSpline[i]>=0.0f)
			{
			dv = Starting[Index].DistToSpline[i];
		    if (dv < 0.0001f) dv = 0.0001f;
			d += dv;
			}
		}

	Tab<float> Inversed;
	for (i=0;i<bshape.SplineCount();i++)
		{
		float nv;
		float dv;

		if (Starting[Index].DistToSpline[i]>=0.0f)
			{
			dv = Starting[Index].DistToSpline[i];
			if (dv < 0.0001f) dv = 0.0001f;

			nv = d/dv;
			Inversed.Append(1,&nv,1);
			}
		else
			{	
			nv = 0.0f;
			Inversed.Append(1,&nv,1);
			}

		}

	d = 0.0f;

	for (i=0;i<bshape.SplineCount();i++)
		{
		d += Inversed[i];
		}

//	DebugPrint("Strand %d \n",Index);
	for (i=0;i<bshape.SplineCount();i++)
		{
		if (Starting[Index].DistToSpline[i]>=0.0f)
			{
			p = (Inversed[i]/d);
			Starting[Index].PercentControl.Append(1,&p,1);
			}
		else
			{
			p = 0.0f;
			Starting[Index].PercentControl.Append(1,&p,1);
			}
//	DebugPrint("   Spline %d Percent %f\n",i,p);

		}

	}


//Build Spline Velocity Data for this frame all at once
float d = 0.0f;
float StartPer, PrevPer;
int life;
int UseAbsolute=1;



pblock->GetValue(PB_ABSOLUTE_DIRECTION,t,UseAbsolute,FOREVER);








int CActive,CFrame;
int stepSize;

stepSize = GetTicksPerFrame();
pblock->GetValue(PB_CUTTERFRAME,t,CFrame,FOREVER);

pblock->GetValue(PB_CUTTERACTIVE,t,CActive,FOREVER);


pblock->GetValue(PB_SEGCOUNT,t,life,FOREVER);
if (TestAFlag(A_RENDER)) 
		life = life *2;



//allocate data spline velocity data struct

SplineVelocityList.ZeroCount();

SplineData TData;
TData.Velocities.ZeroCount();

for (Index = 0;Index<bshape.SplineCount();Index++)
  {

  SplineVelocityList.Append(1,&TData,1);
  for (int CSeg = 0;CSeg<life;CSeg++)
	{
	StartPer = (float)(CSeg)/(float)(life);
	PrevPer = (float)(CSeg-1)/(float)(life);

 
//put in here checks for Cutters active and then used % of spline used

//get vector for emmitter vertex;
	Point3 SplineVector,FirstPoint;
	Point3 PrevPoint,CurrentPoint;
	PrevPoint.x = 0.0f;
	PrevPoint.y = 0.0f;
	PrevPoint.z = 0.0f;
	CurrentPoint.x = 0.0f;
	CurrentPoint.y = 0.0f;
	CurrentPoint.z = 0.0f;


//rotated spline start and end

	Spline3D *spline = bshape.GetSpline(Index);
	FirstPoint = spline->InterpCurve3D(0.0f);
	PrevPoint =  spline->InterpCurve3D(PrevPer) - FirstPoint;
	CurrentPoint = spline->InterpCurve3D(StartPer) - FirstPoint;


	PrevPoint =	PrevPoint * STM;
	CurrentPoint = CurrentPoint * STM;

	CurrentPoint = CurrentPoint - PrevPoint;

	SplineVelocityList[Index].Velocities.Append(1,&CurrentPoint,1);
	
   }

 }

SplineStartingVertsTransformed.ZeroCount();
Point3 PTemp;
for (Index = 0;Index<bshape.SplineCount();Index++)
  {
   PTemp = Normalize(VectorTransform(ETM,SplineStartingVerts[Index]));
   SplineStartingVertsTransformed.Append(1,&PTemp,1);
  }



StartingVelTransFormed.ZeroCount();

for (Index = 0;Index<NumberStrands;Index++)
  {
  PTemp = Normalize(VectorTransform(ETM,Starting[Index].vel));
  StartingVelTransFormed.Append(1,&PTemp,1);
  }


}
